package com.fr.plugin.decision.api.bean.consanguinity;

import com.finebi.burger.api.bean.configuration.table.field.label.FieldLabelBean;
import com.finebi.common.exception.FineEngineException;
import com.finebi.dashboard.api.bean.report.consanguinity.*;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.plugin.decision.api.utils.DirectServiceUtils;
import com.fr.stable.StringUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/9/24 18:07
 * @Description TODO
 * @Version 10.0
 **/
public class DetailReportConsanguinityBean extends ReportConsanguinityBean {

    private BaseInfo info;
    private final String SPECIAL_CHAR_REGEX = "</?[^><]*>";

    public DetailReportConsanguinityBean() {
    }

    public DetailReportConsanguinityBean(ReportConsanguinityBean report) {
        setReportId(report.getReportId());
        setReportName(report.getReportName());
        if (report.getWidgets() != null) {
            setWidgets(report.getWidgets().stream().map(
                    widget -> {
                        // 去除<p></p>
                        widget.setWidgetName(widget.getWidgetName().replaceAll(SPECIAL_CHAR_REGEX, ""));
                        if (widget.getTables() != null) {
                            widget.setTables(widget.getTables().stream().map(this::addTableInfo).collect(Collectors.toList()));
                        }
                        return widget;
                    }
            ).collect(Collectors.toList()));
        }
        setInfo(APIService.getInstance().getTemplateInfo(report.getReportId()));
    }

    public BaseInfo getInfo() {
        return info;
    }

    public void setInfo(BaseInfo info) {
        this.info = info;
    }

    private TableConsanguinityBean addTableInfo(TableConsanguinityBean bean) {
        try {
            List<FieldLabelBean> fieldLabelBeanList = DirectServiceUtils.getTableService().getTableFieldLabels(bean.getTableName());
            AbstractTableConsanguinityBean result;
            switch (bean.getType()) {
                case 1:
                    result = new XDataBaseTableConsanguinityBean((DataBaseTableConsanguinityBean) bean); break;
                case 2:
                    result =  new XSqlTableConsanguinityBean((SqlTableConsanguinityBean) bean); break;
                case 3:
                    result =  new XExcelTableConsanguinityBean((ExcelTableConsanguinityBean) bean); break;
                case 4:
                    result =  new XAnalysisTableConsanguinityBean((AnalysisTableConsanguinityBean) bean);
                    ((XAnalysisTableConsanguinityBean) result).setFatherTables(((AnalysisTableConsanguinityBean) bean).getFatherTables()
                            .stream()
                            .map(this::addTableInfo)
                            .collect(Collectors.toSet()));
                    break;
                case 5:
                    result =  new XServerTableConsanguinityBean((ServerTableConsanguinityBean) bean); break;
                    default:
                        throw new RuntimeException("Unrecognized TableConsanguinity type " + bean.getType());
            }
            result.setFields(bean.getFields().stream().map(field -> addFieldLabel(field, fieldLabelBeanList)).collect(Collectors.toList()));
            return result;
        } catch (FineEngineException e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    /*
    给列对象添加上标签值
     */
    public XFieldConsanguinityBean addFieldLabel(FieldConsanguinityBean bean, List<FieldLabelBean> fieldLabelBeanList) {
        XFieldConsanguinityBean xField = new XFieldConsanguinityBean(bean);
        fieldLabelBeanList.stream().peek(fieldLabelBean -> fieldLabelBean.getContent().stream().peek(contentBean -> {
            if (StringUtils.equals(contentBean.getId(), bean.getFieldId())) {
                xField.setLabel(fieldLabelBean.getValue());
                return;
            }
        }).count()).count();
        return xField;
    }
}
