package com.fr.plugin.decision.api.controller.bi;

import com.finebi.burger.api.bean.configuration.consanguinity.TableConsanguinityInfo;
import com.finebi.burger.api.bean.configuration.group.SimpleGroupBean;
import com.finebi.burger.api.bean.configuration.pack.SimplePackageBean;
import com.finebi.burger.api.bean.configuration.table.AnalysisTableBean;
import com.finebi.burger.api.bean.configuration.table.BusinessTableBean;
import com.finebi.burger.api.bean.configuration.table.SqlTableBean;
import com.finebi.burger.api.bean.configuration.table.etl.operator.OperatorBean;
import com.finebi.burger.api.bean.configuration.table.etl.operator.SelectFieldBean;
import com.finebi.burger.api.bean.configuration.table.field.FineBusinessField;
import com.finebi.burger.api.bean.configuration.table.field.label.FieldLabelSettingBean;
import com.finebi.burger.api.bean.configuration.table.response.TableSaveResultBean;
import com.finebi.burger.api.request.configuration.consanguinity.TableUsageRequestBean;
import com.finebi.burger.api.request.configuration.table.TableInfoUpdateRequestBean;
import com.finebi.burger.api.request.configuration.table.TableMovePackageRequestBean;
import com.finebi.burger.api.request.configuration.table.etl.SimpleAnalysisTableAddRequestBean;
import com.finebi.burger.api.response.configuration.table.SimpleTableInfoBean;
import com.finebi.burger.api.response.configuration.table.SimpleTableNameResponseBean;
import com.finebi.burger.impl.helper.pack.FinePackageHelper;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.authority.structure.GroupPackNode;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.table.AbstractFineTable;
import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.common.authority.api.FineUserUtils;
import com.finebi.common.exception.FineEngineException;
import com.finebi.common.exception.platform.FineAuthorityExecuteException;
import com.finebi.dashboard.api.bean.report.ReportBean;
import com.finebi.dashboard.api.bean.widget.WidgetBean;
import com.finebi.dashboard.api.bean.widget.table.TableWidgetBean;
import com.finebi.dashboard.impl.helper.FineDashBoardUtils;
import com.finebi.direct.burger.excel.ExcelServiceFactory;
import com.finebi.direct.burger.impl.service.helper.table.FineTableUtils;
import com.finebi.direct.web.action.v5.conf.DirectConfAction;
import com.finebi.foundation.utils.ServiceFieldIdUtils;
import com.finebi.utils.CompareUtils;
import com.fr.decision.AdaptUtils;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.FinePathVariable;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.bean.pack.BiPackageTableFieldBean;
import com.fr.decision.webservice.v10.login.TokenResource;
import com.fr.decision.webservice.v10.pack.BiPackageService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONObject;
import com.fr.json.JSONObjectWriter;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.FineRespondNotNull;
import com.fr.plugin.decision.api.bean.consanguinity.table.XTableConsanguinityInfo;
import com.fr.plugin.decision.api.bean.table.AddTableResult;
import com.fr.plugin.decision.api.bean.table.TableBaseBean;
import com.fr.plugin.decision.api.direct.DirectPackageGroupServiceForHW;
import com.fr.plugin.decision.api.direct.service.XAnalysisTableServiceHelper;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.plugin.decision.api.service.ConfService;
import com.fr.plugin.decision.api.service.PackageTableSearcher;
import com.fr.stable.CommonUtils;
import com.fr.stable.StringUtils;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Controller
@RequestMapping("/api/v5/plugin/direct")
//未登录回跳转至登录页，对接口返回值来说不明确，优化下返回值
//@LoginStatusChecker(required = false)
public class DirectConfResource extends DirectConfAction {

    /**
     * 新建分组，新建->重命名->？失败删除
     *
     * @param req
     * @param groupId 新建分组父节点ID
     * @param name    新建分组名称
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/groups/{groupId}"},
            method = {RequestMethod.POST}
    )
    public Response addGroup(HttpServletRequest req,
                             @FinePathVariable("groupId") String groupId,
                             @RequestParam("name") String name) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        SimpleGroupBean simpleGroup = getGroupService().addEmptyGroup(parseNoGroupId(groupId), authorityUser);
        if (!StringUtils.equals(name, simpleGroup.getName())) {
            boolean result = getGroupService().updateGroupName(simpleGroup.getId(), name, authorityUser);
            if (!result) {
                getGroupService().deleteGroupInfo(simpleGroup.getId(), authorityUser);
                return Response.error("addGroup", "Group name is illegal or duplicate!");
            }
            simpleGroup.setName(name);
        }
        return Response.ok(simpleGroup);
    }

    /**
     * 分组重命名
     *
     * @param req
     * @param groupId 分组id
     * @param name    分组新名称
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/groups/{groupId}/rename"},
            method = {RequestMethod.PUT}
    )
    public Response renameGroup(HttpServletRequest req,
                                @FinePathVariable("groupId") String groupId,
                                @RequestParam("name") String name) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        boolean result = getGroupService().updateGroupName(groupId, name, authorityUser);
        return result ? Response.success() : Response.error("renameGroup", "group name is illegal or duplicate!");
    }

    /**
     * 删除分组
     *
     * @param groupId 分组id
     * @param req
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/groups/{groupId}"},
            method = {RequestMethod.DELETE}
    )
    public Response deleteGroup(@FinePathVariable("groupId") String groupId, HttpServletRequest req) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        if (!authorityUser.isGroupPackManageable(GroupPackNode.group(groupId))) {
            return Response.error("deleteGroup", "User does not have permission!");
        }
        boolean result = getGroupService().deleteGroupInfo(groupId, authorityUser);
        return result ? Response.success() : Response.error("deleteGroup", "Delete group failed, please check the log file!");
    }

    /**
     * 移动分组
     *
     * @param req
     * @param groupId 分组id
     * @param pId
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/groups/{groupId}/move"},
            method = {RequestMethod.PUT}
    )
    public Response moveGroup(HttpServletRequest req,
                              @FinePathVariable("groupId") String groupId,
                              @RequestParam("pId") String pId) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        boolean result = getGroupService().moveGroup(groupId, pId, authorityUser);
        return result ? Response.success() : Response.error("moveGroup", "Move group failed, please check the log file!");
    }

    /**
     * 获取用户所有分组和业务包结构
     *
     * @param req
     * @return
     * @throws Exception
     */
    /*@ResponseBody
    @RequestMapping(
            value = {"/groups"},
            method = {RequestMethod.GET}
    )
    public Response getAllGroupInfo(HttpServletRequest req) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        return Response.ok(getGroupService().getAllGroupInfo(authorityUser));
    }*/

    /**
     * 获取指定分组ID下业务包和分组结构
     * 添加下业务包下数据集的返回
     *
     * @param req
     * @param pId 分组ID
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/groups"},
            method = {RequestMethod.GET}
    )
    public Response getGroupInfoById(HttpServletRequest req,
                                     @RequestParam(value = "pId", required = false) String pId,
                                     @RequestParam(required = false, value = "st") String st,
                                     @RequestParam(required = false, value = "et") String et) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        List<SimpleGroupBean> list;
        if (StringUtils.isBlank(pId)) {
            list = getGroupService().getAllGroupInfo(authorityUser);
        } else {
            list = DirectPackageGroupServiceForHW.getAllGroupInfo(pId, authorityUser);
        }
        ConfService.getInstance().addTableByPackage(list, authorityUser, st, et);
        return Response.ok(list);
    }



    /**
     * 新建业务包
     *
     * @param groupId 分组id
     * @param req
     * @param name    分组名称
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{groupId}"},
            method = {RequestMethod.POST}
    )
    public Response addPackage(@FinePathVariable("groupId") String groupId, HttpServletRequest req, @RequestParam("name") String name) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        FineBusinessPackage newPackage = getPackageService().addNewPackage(parseNoGroupId(groupId), authorityUser);
        SimplePackageBean simplePackage = FinePackageHelper.createSimplePackageBean(newPackage, authorityUser.getUser().getHelper());
        if (!StringUtils.equals(simplePackage.getName(), name)) {
            boolean result = getPackageService().updatePackageName(simplePackage.getId(), name, authorityUser);
            if (!result) {
                getPackageService().removePackage(simplePackage.getId(), authorityUser);
                return Response.error("addPackage", "Package name is illegal or duplicate!");
            }
            simplePackage.setName(name);
        }
        return Response.ok(simplePackage);
    }

    /**
     * 重命名业务包
     *
     * @param packId 业务包id
     * @param req
     * @param name   分组名称
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{packId}/rename"},
            method = {RequestMethod.PUT}
    )
    public Response renamePackage(@FinePathVariable("packId") String packId, HttpServletRequest req, @RequestParam("name") String name) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        boolean result = getPackageService().updatePackageName(packId, name, authorityUser);
        return result ? Response.success() : Response.error("renamePackage", "Package name is illegal or duplicate!");
    }

    /**
     * 删除业务包
     *
     * @param packId
     * @param req
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{packId}"},
            method = {RequestMethod.DELETE}
    )
    public Response deletePackage(@FinePathVariable("packId") String packId, HttpServletRequest req) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        if (!authorityUser.isGroupPackManageable(GroupPackNode.pack(packId))) {
            return Response.error("deleteGroup", "User does not have permission!");
        }
        boolean result = getPackageService().removePackage(packId, authorityUser);
        return result ? Response.success() : Response.error("deletePackage", "Delete package failed, please check the log file!");
    }

    /**
     * 获取业务包中数据集信息
     *
     * @param packId
     * @param req
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{packId}"},
            method = {RequestMethod.GET}
    )
    public Response getPackageTables(@FinePathVariable("packId") String packId, HttpServletRequest req) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        List result = new ArrayList();
        for (String pack : packId.split(",")) {
            result.add(getPackageService().getPackageTables(pack, authorityUser, false));
        }
        return Response.ok(result);
    }

    /**
     * 获取业务包中数据集基础信息
     *
     * @param packId
     * @param req
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{packId}/simple"},
            method = {RequestMethod.GET}
    )
    public Response getPackageSimpleTables(@FinePathVariable("packId") String packId, HttpServletRequest req) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        List result = new ArrayList();
        for (String pack : packId.split(",")) {
            result.add(DirectPackageGroupServiceForHW.getPackageTables(pack, authorityUser, false));
        }
        return Response.ok(result);
    }

    /**
     * 移动业务包
     *
     * @param req
     * @param pId    业务包id
     * @param packId 业务包移动至id
     * @return
     * @throws FineEngineException
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/{packId}/move"},
            method = {RequestMethod.PUT}
    )
    public Response movePackage(HttpServletRequest req,
                                @RequestParam("pId") String pId,
                                @FinePathVariable("packId") String packId) throws FineEngineException {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        boolean result = getGroupService().modifyGroup(pId, packId, authorityUser);
        return result ? Response.success() : Response.error("movePackage", "Move package failed, please check the log file!");
    }

    /**
     * 搜索分组和业务包
     *
     * @param req
     * @param keyword 关键字
     * @param pId     来源，分组/业务包ID
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/packs/search"},
            method = {RequestMethod.GET}
    )
    public Response searchGroupAndTable(HttpServletRequest req,
                                        @RequestParam("keyword") String keyword,
                                        @RequestParam("pId") String pId) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        PackageTableSearcher searcher = new PackageTableSearcher(getPackageService());
        return Response.ok(searcher.searchResult(authorityUser, keyword, pId));
    }

    /**
     * 批量添加SQL数据集，并预览列信息，预览报错时回滚删除数据集
     * {@link com.finebi.direct.web.action.v5.conf.DirectBusinessTableAction#saveTable(BusinessTableBean, String, HttpServletRequest)}
     *
     * @return 数据集的所有列信息
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/sql"},
            method = {RequestMethod.POST}
    )
    public Response addSqlTables(@RequestBody List<BusinessTableBean> tableBeans,
                                 HttpServletRequest request,
                                 @RequestParam(value = "skipCheck", required = false) boolean skipCheck) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(request);
        List resultBeanList = new ArrayList<>();
        for (BusinessTableBean tableBean : tableBeans) {
            SqlTableBean sqlTableBean = ((SqlTableBean) tableBean);
            TableSaveResultBean resultBean;
            try {
                resultBean = FineTableUtils.saveTable(sqlTableBean, sqlTableBean.getName(), authorityUser);
            } catch (Exception e) {
                FineLoggerFactory.getLogger().error(e.getMessage(), e);
                resultBeanList.add(Response.error("addSqlTables", e.getMessage()));
                continue;
            }
            if (!skipCheck) {
                try {
                    SimpleTableInfoBean simpleTableInfoBean = ConfService.getInstance().sqlCheck(sqlTableBean, authorityUser);
                    resultBeanList.add(new AddTableResult(resultBean,
                                    simpleTableInfoBean.getFields().stream()
                                            .map(field -> {
                                                field.setId(ServiceFieldIdUtils.createFineFieldId(resultBean.getName(), field.getId()));
                                                return field;
                                            })
                                            .collect(Collectors.toList())
                            )
                    );
                } catch (Exception e) {
                    FineLoggerFactory.getLogger().error(e.getMessage(), e);
                    resultBeanList.add(Response.error("check sql", e.getMessage()));
                    getTableService().removeBusinessTable(sqlTableBean.getName(), authorityUser);
                }
            }
        }
        return Response.ok(resultBeanList);
    }

    /**
     * 批量添加自助数据集
     *
     * @param tableBeans
     * @param request
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/analysis"},
            method = {RequestMethod.POST}
    )
    @Deprecated
    public Response addAnalysisTables(@RequestBody List<BusinessTableBean> tableBeans,
                                      HttpServletRequest request) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(request);
        List resultBeanList = new ArrayList<>();
        for (BusinessTableBean tableBean : tableBeans) {
            ((AnalysisTableBean) tableBean).setEngineType(1);
            tableBean.setOperators(Arrays.asList(new OperatorBean[]{new SelectFieldBean()}));
            FineTableUtils.saveTable(tableBean, tableBean.getName(), authorityUser);
            FineBusinessTable businessTable = FineTableUtils.getTableByNameWithAuth(tableBean.getName(), authorityUser);
            resultBeanList.add(new AddTableResult(businessTable.getName(), businessTable.getTransferName(), businessTable.getFields()));
        }
        return Response.ok(resultBeanList);
    }

    /**
     * 删除直连下的数据集
     *
     * @param tableId
     * @param request
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/{tableId}"},
            method = {RequestMethod.DELETE}
    )
    public Response deleteTable(@FinePathVariable("tableId") String tableId,
                                HttpServletRequest request) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(request);
        ExcelServiceFactory.getService().removeExtractedExcelData(tableId);
        boolean result = getTableService().removeBusinessTable(tableId, authorityUser);
        return result ? Response.success() : Response.error("deleteTable", "Delete table failed, please check the log file!");
    }

    /**
     * 数据集重命名
     *
     * @param updateRequestBean
     * @param tableName         数据集name
     * @param request
     * @return
     * @throws FineEngineException
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/{tableName}/rename"},
            method = {RequestMethod.PUT}
    )
    public Response updateTableInfo(@RequestBody TableInfoUpdateRequestBean updateRequestBean,
                                    @FinePathVariable("tableName") String tableName,
                                    HttpServletRequest request) throws FineEngineException {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(request);
        boolean success = getTableService().updateTableExtraInfo(tableName, updateRequestBean, authorityUser, getTableLockService());
        return success ? Response.success() : Response.error("updateTableInfo", "table name is illegal or duplicate!");
    }

    /**
     * 移动数据集
     *
     * @param tableName
     * @param requestBean
     * @param req
     * @return
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/{tableName}/move"},
            method = {RequestMethod.PUT}
    )
    public Response moveTableData(@FinePathVariable("tableName") String tableName,
                                  @RequestBody TableMovePackageRequestBean requestBean,
                                  HttpServletRequest req) throws FineEngineException {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        boolean success = getPackageService().modifyTablePackage(tableName, requestBean.getNewPackId(), authorityUser);
        return success ? Response.success() : Response.error("moveTableData", "Move table failed, please check the log file!");
    }

    /**
     * 查询数据集列信息 {@link com.fr.decision.webservice.v10.pack.BiPackageService#getTableFields(String, String, boolean)}
     *
     * @param request
     * @param tableId
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/{tableId}/columns"},
            method = {RequestMethod.GET}
    )
    public Response addAnalysisTables(HttpServletRequest request,
                                      @FinePathVariable("tableId") String tableId) throws Exception {
        String currentUserId = UserService.getInstance().getCurrentUserId(request);
        List<BiPackageTableFieldBean> fieldBeanList = BiPackageService.getInstance().getTableFields(
                currentUserId,
                tableId,
                true
        );
        return Response.ok(fieldBeanList);
    }

    /**
     * 校验数据集名称是否存在重复，BI 21.8.27版本支持跨业务包同展示名
     *
     * @param tableBeanList name全局校验 ; transferName packId 校验展示名称
     * @return true不重名， false重名
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/tables/check"},
            method = {RequestMethod.POST}
    )
    public Response checkTableName(@RequestBody List<TableBaseBean> tableBeanList) throws Exception {
        ArrayList<Boolean> result = new ArrayList();
        out:
        for (TableBaseBean table : tableBeanList) {
            List<SimpleTableNameResponseBean> tablesName = getDashboardTableService().getAllTableNamesAndTransferNames();
            for (SimpleTableNameResponseBean simpleTableNameBean : tablesName) {
                if (CompareUtils.isEqual(simpleTableNameBean.getTableName(), table.getName())) {
                    result.add(false);
                    continue out;
                }
            }

            List<SimpleTableNameResponseBean> tableNameResponseBeans = getTableService().getTableTransferNamesByPackId(table.getPackId());
            for (SimpleTableNameResponseBean bean : tableNameResponseBeans) {
                if (CompareUtils.isEqual(bean.getTransferName(), table.getTransferName())) {
                    result.add(false);
                    continue out;
                }
            }
            result.add(true);
        }

        return Response.ok(result);
    }



    /**
     * 空白报表创建组件并自动打开数据集
     *
     * @param request
     * @param response
     * @param tableName 数据集name
     * @param reportId  仪表板id
     * @return
     * @throws Exception
     */
    @ResponseBody
    @RequestMapping(
            value = {"/table/widget/{tableName}"}
    )
    @LoginStatusChecker(tokenResource = TokenResource.COOKIE)
    public String createDashboardAndOpenTable(HttpServletRequest request,
                                              HttpServletResponse response,
                                              @FinePathVariable("tableName") String tableName,
                                              @RequestParam("reportId") String reportId) throws Exception {

        User user = FineUserUtils.getUserByRequestSafe(request);
        ReportBean reportBean = getDashboardService().getReportById(reportId);
        if (reportBean.getWidgets().isEmpty()) {
            TableWidgetBean widgetBean = FineDashBoardUtils.constructEmptyWidget(tableName, APIService.getInstance().constructTableWidget());
            Map<String, WidgetBean> widgetBeanMap = new HashMap();
            widgetBeanMap.put(widgetBean.getWid(), widgetBean);
            reportBean.setWidgets(widgetBeanMap);
            getDashboardService().saveReport(reportBean, user);
        }

        Map<String, String[]> params = request.getParameterMap();
        String paramStr = StringUtils.EMPTY;
        if (!params.isEmpty()) {
            List<String> keyValueParam = new ArrayList<>(params.size());
            params.forEach((key, value) -> {
                if (!StringUtils.equals(key, "reportId")) keyValueParam.add(key + "=" + value[0]);
            });
            if (!keyValueParam.isEmpty())
                paramStr = keyValueParam.stream().reduce((a, b) -> a + "&" + b).get();
        }

        response.setContentType("text/html; charset=GB2312");
        return APIService.getInstance().writeLocalStorageAndOpenWindowScriptStr(user.getId(), tableName, reportId, paramStr);
    }

    /**
     * 获取数据集的基本信息
     *
     * @param tableName
     * @return
     * @throws Exception
     */
    @RequestMapping(
            value = {"/tables/{tableName}/info"},
            method = {RequestMethod.GET}
    )
    @ResponseBody
    public Response getTableInfo(@FinePathVariable("tableName") String tableName) throws Exception {
        return Response.ok(ConfService.getInstance().getTableBaseInfo(tableName));
    }

    /**
     * 处理下异常返回值统一格式只返回 errorCode errorMsg
     *
     * @param ex
     * @return
     */
    @Override
    public FineRespondNotNull exceptionRespond(Exception ex) {
        return new FineRespondNotNull(super.exceptionRespond(ex));
    }

    /**
     * 获取数据集的血缘分析
     *
     * @param req
     * @param tableName
     * @return
     * @throws Exception
     */
    @RequestMapping(
            value = {"/tables/{tableName}/usage"},
            method = {RequestMethod.GET}
    )
    @ResponseBody
    public Response getTableUsage(HttpServletRequest req,
                                  @FinePathVariable("tableName") String tableName) throws Exception {
        AbstractFineTable businessTable = (AbstractFineTable) getTableService().getBusinessTableByName(tableName);
        TableUsageRequestBean usageRequestBean = new TableUsageRequestBean();
        usageRequestBean.setTableName(tableName);
        usageRequestBean.setType(businessTable.getType());
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        TableConsanguinityInfo tableConsanguinityInfo = getTableService().getTableConsanguinityInfo(usageRequestBean, authorityUser);
        return Response.ok(new XTableConsanguinityInfo(tableConsanguinityInfo, businessTable));
    }

    /**
     * 修改数据集列标签
     *
     * @param req
     * @param tableName
     * @param labelSettingBean
     * @return
     * @throws Exception
     */
    @RequestMapping(
            value = {"/tables/{tableName}/field/label"},
            method = {RequestMethod.POST}
    )
    @ResponseBody
    public Response updateFieldLabel(HttpServletRequest req,
                                     @FinePathVariable("tableName") String tableName,
                                     @RequestBody FieldLabelSettingBean labelSettingBean) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);
        labelSettingBean.getLabels().stream().peek(fieldLabelBean -> fieldLabelBean.setId(CommonUtils.getRandomStr(16))).count();
        boolean result = getTableService().setFieldLabelSetting(tableName, labelSettingBean, authorityUser);
        return result ? Response.success() : Response.error("updateFieldLabel", "Update field label failed, please check the log file!!");
    }

    /**
     * 新建自助数据集指定来源
     * @param tableBeans 列名为空默认全选
     * @param request
     * @return
     * @throws Exception
     */
    @RequestMapping(
            value = "/tables/analysis/source",
            method = RequestMethod.POST
    )
    @ResponseBody
    public Response addSimpleAnalysisTable(@RequestBody List<SimpleAnalysisTableAddRequestBean> tableBeans, HttpServletRequest request) throws Exception {
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(request);
        ArrayList result = new ArrayList(tableBeans.size());
        for (SimpleAnalysisTableAddRequestBean tableBean : tableBeans) {
            try {
                /*if (tableBean.getSourceTableInfo().getFieldNames() == null || tableBean.getSourceTableInfo().getFieldNames().isEmpty()) {
                    String source = tableBean.getSourceTableInfo().getTableName();
                    List<FineBusinessField> fields = AdaptUtils.getBIEngineProviderByEngineType("direct").getTableFieldsWithAuth(Arrays.asList(source), null).get(source);
                    tableBean.getSourceTableInfo().setFieldNames(
                            fields.stream().map(e-> e.getName()).collect(Collectors.toList())
                    );
                }*/
                result.add(ConfService.getInstance().addSimpleAnalysisTable(tableBean, authorityUser));
            } catch (Exception e) {
                result.add(Response.error(tableBean.getTableName(), e.getMessage()));
            }
        }

        return Response.ok(result);
    }
}
