package com.fr.plugin.decision.api.controller.bi;

import com.finebi.burger.api.bean.configuration.group.SimpleGroupBean;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.exception.FineEngineException;
import com.finebi.dashboard.api.bean.report.ReportDirBean;
import com.finebi.dashboard.api.response.report.dashboard.ReportIndexBean;
import com.finebi.direct.web.action.v5.conf.DirectConfAction;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.base.constant.AuthorityStaticItemId;
import com.fr.decision.authority.data.Department;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.annotation.VisitRefer;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.batch.MoveBean;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.RequestBody;
import com.fr.third.springframework.web.bind.annotation.RequestMapping;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/11/22 14:57
 * @Description TODO
 * @Version 10.0
 **/
@Controller
@RequestMapping(value = "/api/v5/plugin/combine")
public class GroupResources extends DirectConfAction {

    public static final String ANALYSIS_GROUP_NAME = "我的自助数据集";
    public static final String ANALYSIS_SHARA_NAME = "分享给我的自助数据集";

    @RequestMapping(
            value = "/move",
            method = RequestMethod.POST
    )
    @ResponseBody
    @LoginStatusChecker
    @VisitRefer(refer = AuthorityStaticItemId.DEC_MANAGEMENT_USER_ADD_DEPARTMENT_ID)
    public Response moveDatasetAndTemplate2TenantByDepartment(HttpServletRequest request,
                                                              HttpServletResponse response,
                                                              @RequestBody List<MoveBean> moveBeans) {
        return Response.ok(moveBeans.stream()
                .map(moveBean -> {
                    try {
                        moveDatasetAndTemplate2Tenant(moveBean);
                        return Response.success();
                    } catch (Exception e) {
                        FineLoggerFactory.getLogger().error(e.getMessage(), e);
                        return Response.error("moveDatasetAndTemplate2Tenant", e.getMessage());
                    }
                })
                .collect(Collectors.toList())
        );
    }

    private void moveDatasetAndTemplate2Tenant(MoveBean moveBean) throws Exception {
        String userId = APIService.getInstance().transferUser2Id(moveBean.getUser());
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(userId);
        if (moveBean.getFolderId() != null) {
            createAndMoveTemplateFolder(userId, moveBean.getFolderId());
        }
        if (moveBean.getGroupId() != null) {
            createAndMoveTableData(userId, moveBean.getGroupId(), authorityUser);
        }
    }

    /**
     * 获取用户租户名
     *
     * @param userId
     * @return
     * @throws Exception
     */
    private String getUserTenantName(String userId) throws Exception {
        List<Department> departmentList = AuthorityContext.getInstance().getDepartmentController().findByUser(userId, QueryFactory.create());
        if (!departmentList.isEmpty()) {
            return departmentList.get(0).getName();
        }
        throw new RuntimeException("User not exist in department!");
    }

    /**
     * 创建并移动仪表板
     *
     * @param userId
     * @param folderId
     * @throws Exception
     */
    private void createAndMoveTemplateFolder(String userId, String folderId) throws Exception {
        List<ReportIndexBean> flFolder = getDashboardService().getDashBoardList(createReportIndex(userId), userId);
        List<ReportIndexBean> tenantFolders = flFolder.stream()
                .filter(reportIndex -> reportIndex.isFolder() && StringUtils.equals(reportIndex.getId(), folderId))
                .collect(Collectors.toList());
        ReportDirBean tenantDirBean = new ReportDirBean();
        if (tenantFolders.isEmpty()) {
            /*ReportDirBean createBean = new ReportDirBean();
            createBean.setName(tenant);
            createBean.setFolder(true);
            ReportIndexBean createReport = getDashboardService().createNewReportFolder(createBean, userId);
            createBean.setReportId(createReport.getId());
            createBean.setNewName(tenant);
            getDashboardService().renameReportDirBean(createBean, userId);
            tenantDirBean.setReportId(createReport.getId());*/
            throw new RuntimeException("Move Template Folder to " + folderId + " not exist!");
        } else {
            tenantDirBean.setReportId(tenantFolders.get(0).getId());
            flFolder.removeAll(tenantFolders);
        }
        getDashboardService().moveReports(flFolder.stream().map(e -> createReportIndex(e.getId())).collect(Collectors.toList()), tenantDirBean, userId);
    }

    private ReportDirBean createReportIndex(String reportId) {
        ReportDirBean reportDirBean = new ReportDirBean();
        reportDirBean.setReportId(reportId);
        return reportDirBean;
    }

    /**
     * 创建并移动我的自助数据集目录
     *
     * @param userId
     * @param groupId
     * @param authorityUser
     * @throws Exception
     */
    private void createAndMoveTableData(String userId, String groupId, UserAuthority authorityUser) throws Exception {
        // Optional<GroupTreeNodeVO> optional = CellCreator.getCellProvider().packageCell().getSubGroupPackageTreeByGroupId(analysisGroupId, authorityUser);
        List<SimpleGroupBean> rootGroup = getGroupService().getAllGroupInfo(authorityUser);
        List<SimpleGroupBean> analysisGroupList = rootGroup
                .stream()
                .filter(e -> e.isMyAnalysis() && StringUtils.equals(e.getName(), ANALYSIS_GROUP_NAME))
                .collect(Collectors.toList());
        if (!analysisGroupList.isEmpty()) {
            SimpleGroupBean analysisGroup = analysisGroupList.get(0);
            String analysisGroupId = analysisGroup.getId();
            List<SimpleGroupBean> flGroup = analysisGroup.getGroups();
            List<SimpleGroupBean> tenantGroupList = flGroup
                    .stream()
                    .filter(e -> StringUtils.equals(e.getId(), groupId))
                    .collect(Collectors.toList());
            String tenantGroupId = StringUtils.EMPTY;
            if (tenantGroupList.isEmpty()) {
                /*SimpleGroupBean tenantGroup = getGroupService().addEmptyGroup(analysisGroupId, authorityUser);
                getGroupService().updateGroupName(tenantGroup.getId(), tenant, authorityUser);
                tenantGroupId = tenantGroup.getId();*/
                throw new RuntimeException("Move Analysis Group to " + groupId + " not exist!");
            } else {
                tenantGroupId = tenantGroupList.get(0).getId();
                flGroup.removeAll(tenantGroupList);
            }

            String finalTenantGroupId = tenantGroupId;
            flGroup.stream()
                    .peek(e -> moveGroup(e.getId(), finalTenantGroupId, authorityUser))
                    .collect(Collectors.toList());
            analysisGroup.getPacks().stream()
                    .filter(e -> !StringUtils.equals(e.getName(), ANALYSIS_SHARA_NAME))
                    .peek(e -> movePackage(e.getId(), finalTenantGroupId, authorityUser))
                    .collect(Collectors.toList());
        }
    }

    private void moveGroup(String groupId, String pId, UserAuthority authorityUser) {
        try {
            getGroupService().moveGroup(groupId, pId, authorityUser);
        } catch (FineEngineException e1) {
            throw new RuntimeException(e1);
        }
    }

    private void movePackage(String packageId, String pId, UserAuthority authorityUser) {
        try {
            getGroupService().modifyGroup(pId, packageId, authorityUser);
        } catch (FineEngineException e1) {
            throw new RuntimeException(e1);
        }
    }
}
