package com.fr.plugin.decision.api.controller.bi;

import com.finebi.foundation.api.bean.handover.ResourceHandoverBean;
import com.finebi.foundation.api.bean.handover.ResourceHandoverResultBean;
import com.fr.decision.authority.data.User;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.v10.resource.handover.BIResourceHandoverService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @Author JianYe.Wang
 * @Data 2021/9/24 16:02
 * @Description TODO
 * @Version 10.0
 **/
@Controller
@RequestMapping("/api/v5/plugin/resource/handover")
public class HandoverController {

    public HandoverController() {
    }

    @RequestMapping(
            value = {"/acquire"},
            method = {RequestMethod.GET}
    )
    @ResponseBody
    public Response getResourceForHandover(@RequestParam(value = "username") String username, HttpServletRequest request) throws Exception {
        User user = UserService.getInstance().getUserByUserName(username);
        if (user == null) {
            return Response.error("GetResourceForHandover", "User not exist " + username);
        }
        return Response.ok(BIResourceHandoverService.getInstance().getResourceForHandover(
                UserService.getInstance().getUserByRequest(request).getId(),
                user.getId())
        );
    }

    @RequestMapping(
            value = {"/start"},
            method = {RequestMethod.POST}
    )
    @ResponseBody
    public Response handoverResource(@RequestBody ResourceHandoverBean handoverBean, HttpServletRequest request) throws Exception {
        try {
            User sender = UserService.getInstance().getUserByUserName(handoverBean.getSender());
            User accepter = UserService.getInstance().getUserByUserName(handoverBean.getAccepter());
            if (sender == null || accepter == null) {
                return Response.error("HandoverResource", "User not exist " + handoverBean.getSender() + " " + handoverBean.getAccepter());
            }
            handoverBean.setSender(sender.getId());
            handoverBean.setAccepter(accepter.getId());
            ResourceHandoverResultBean handoverResultBean = BIResourceHandoverService.getInstance().handoverResource(
                    UserService.getInstance().getUserByRequest(request).getId(),
                    handoverBean,
                    request.getSession().getId());
            if (handoverResultBean.isSuccess()) return Response.success();
            return Response.error("HandoverResource", "handover failed, detail message see log file!");
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            return Response.error("HandoverResource", e.getMessage());
        }
    }
}
