package com.fr.plugin.decision.api.controller.platform;

import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.provider.api.decision.FineDecisionAdaptEngineProvider;
import com.fr.decision.AdaptUtils;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.bean.data.transfer.importation.ImportDataParameterBean;
import com.fr.decision.webservice.bean.data.transfer.importation.TransferImportDisplayDataBean;
import com.fr.decision.webservice.bean.data.transfer.importation.TransferImportResultBean;
import com.fr.decision.webservice.v10.data.transfer.DataTransferService;
import com.fr.general.http.HttpRequest;
import com.fr.json.JSONObject;
import com.fr.plugin.decision.api.bean.transfer.DashboardBean;
import com.fr.plugin.decision.api.bean.transfer.TableBean;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.plugin.decision.api.utils.HttpToolbox;
import com.fr.third.org.apache.http.entity.mime.MultipartEntityBuilder;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.RequestMapping;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.third.springframework.web.bind.annotation.RequestParam;
import com.fr.third.springframework.web.bind.annotation.ResponseBody;
import com.fr.third.springframework.web.multipart.MultipartFile;
import com.fr.third.springframework.web.multipart.MultipartHttpServletRequest;
import com.fr.third.springframework.web.multipart.MultipartResolver;
import com.fr.third.springframework.web.multipart.commons.CommonsMultipartResolver;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/10/21 17:16
 * @Description TODO
 * @Version 10.0
 **/
@Controller
@RequestMapping("/api/v5/plugin/platform")
public class PlatformController {

    @RequestMapping(
            value = "/transference/packages/tree",
            method = RequestMethod.GET
    )
    @ResponseBody
    public Response getPackageTree(@RequestParam(value = "engineType", required = false) String engineType) throws Exception {
        return Response.ok(APIService.getInstance().getAllPackagesTree(engineType));
    }


    @RequestMapping(
            value = "/transference/import",
            method = RequestMethod.POST
    )
    @ResponseBody
    public Response dataTransferImport(HttpServletRequest request) throws Exception {
        /*Response response = new HttpResponse();
        AttachmentService.getInstance().uploadAttach(request, response, 32, 32, false);*/
        JSONObject uploadFile = new JSONObject(uploadFile(request));
        List<TransferImportDisplayDataBean> displayDataBeans = DataTransferService.getInstance().displayImportingData(uploadFile.getString("attach_id"));
        ImportDataParameterBean importDataParameterBean = new ImportDataParameterBean();
        importDataParameterBean.setAttachId(uploadFile.getString("attach_id"));
        importDataParameterBean.setImportDataBeans(displayDataBeans.toArray(new TransferImportDisplayDataBean[0]));
        List<TransferImportResultBean> transferImportResultBeans = DataTransferService.getInstance().importEntryData(importDataParameterBean);
        return Response.ok(transferImportResultBeans.stream().map(e -> {
            try {
                if (e.getEntityType() == 201) {
                    String tableName = e.getDependencyPath().get(0);
                    FineDecisionAdaptEngineProvider provider = AdaptUtils.getBIEngineProviderByTable(tableName);
                    String packId = provider.getPackageByTableName(tableName).getId();
                    FineBusinessTable businessTable = provider.getTableByName(tableName);
                    /*String packId = getPackageService().getPackageIdByTableName(tableName);
                    FineBusinessTable businessTable = getTableService().getBusinessTableByName(tableName);*/
                    return new TableBean(tableName, businessTable.getTransferName(), packId, businessTable.getType());
                } else if (e.getEntityType() == 0) {
                    String templatePath = e.getDependencyPath().get(0);
                    String templateId = templatePath.substring(templatePath.lastIndexOf("/") + 1, templatePath.length() - 4);
                    return new DashboardBean(e.getFullPath(), templateId);
                }
                return e;
            } catch (Exception e1) {
                throw new RuntimeException(e1);
            }
        }).collect(Collectors.toList()));
    }

    private String uploadFile(HttpServletRequest request) throws Exception {
        MultipartResolver resolver = new CommonsMultipartResolver(request.getSession().getServletContext());
        MultipartHttpServletRequest multipart = resolver.resolveMultipart(request);
        HashMap header = new HashMap();
        header.put("Content-Type", "multipart/form-data; boundary=AjaxUploadBoundary" + System.currentTimeMillis());
        header.put("Authorization", request.getHeader("Authorization"));
        MultipartEntityBuilder builder = MultipartEntityBuilder.create();
        File uploadFile = transferToFile(multipart.getFile("FileData"));
        builder.addBinaryBody("FileData", uploadFile);
        String result = HttpToolbox.executeAndParse(HttpRequest.custom()
                .url(request.getRequestURL().toString().split("/api/v5/plugin/")[0] + "/v10/attach/upload?width=32&height=32&filename=resource.zip")
                .post(builder.build())
                .headers(header).build());
        uploadFile.delete();
        return result;
    }

    private File transferToFile(MultipartFile multipartFile) throws IOException {
        //选择用缓冲区来实现这个转换即使用java 创建的临时文件 使用 MultipartFile.transferto()方法 。
        File file = null;
        String originalFilename = multipartFile.getOriginalFilename();
        String[] filename = originalFilename.split("\\.");
        file = File.createTempFile(filename[0], filename[1]);
        multipartFile.transferTo(file);
        file.deleteOnExit();
        return file;
    }


}
