package com.fr.plugin.decision.api.controller.platform;

import com.fr.decision.authority.base.constant.AuthorityStaticItemId;
import com.fr.decision.webservice.Response;
import com.fr.decision.webservice.annotation.LoginStatusChecker;
import com.fr.decision.webservice.annotation.RoleCheckerType;
import com.fr.decision.webservice.annotation.VisitRefer;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.bean.user.UserUpdateBean;
import com.fr.decision.webservice.exception.general.NoPrivilegeException;
import com.fr.decision.webservice.impl.user.type.UserProductType;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.plugin.decision.api.bean.user.UserUpdateDepRoleBean;
import com.fr.plugin.decision.api.service.APIService;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.stereotype.Controller;
import com.fr.third.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @Author JianYe.Wang
 * @Data 2021/7/22 9:24
 * @Description TODO
 * @Version 10.0
 **/
@Controller
@RequestMapping(value = "/api/{version}/plugin/user")
@LoginStatusChecker
@VisitRefer(refer = AuthorityStaticItemId.DEC_MANAGEMENT_USER_ID)
public class UserController {

    @RequestMapping(method = RequestMethod.POST)
    @ResponseBody
    //@DecisionUserChecker(targetIndex = 4)
    public Response addUser(HttpServletRequest req,
                            HttpServletResponse res,
                            @PathVariable("version") String version,
                            @RequestBody UserBean userBean) throws Exception {

        UserService.getInstance().addUser(userBean);
        return Response.ok(UserService.getInstance().getUser(UserService.getInstance().getCurrentUserId(userBean.getUsername())));
    }

    /**
     * 基于用户批量更新更新 部门职位、角色信息
     *
     * @param request
     * @param response
     * @param bean
     * @return
     * @throws Exception
     */
    @RequestMapping(value = {"/info"}, method = {RequestMethod.PUT})
    //@VisitRefer(refer = AuthorityStaticItemId.DEC_MANAGEMENT_USER_EDIT_INFO_ID)
    @ResponseBody
    public Response updateUserDepRoleInfo(HttpServletRequest request,
                                          HttpServletResponse response,
                                          @RequestBody UserUpdateDepRoleBean bean) throws Exception {
        String userId = UserService.getInstance().getCurrentUserId(request);
        if (!RoleCheckerType.USER.checkAuthority(userId, bean.getUserId())) {
            throw new NoPrivilegeException(userId, bean.getUserId());
        } else {
            //UserService.getInstance().editUserInfo(userBean);
            APIService.getInstance().updateUserDepRoleInfo(userId, bean);
            return Response.success();
        }
    }

    @RequestMapping(value = {"/info"}, method = RequestMethod.GET)
    @ResponseBody
    public Response getUserInfoByName(HttpServletRequest request,
                                      @RequestParam(value = "username", required = false) String username,
                                      @RequestParam(value = "userId", required = false) String userId) throws Exception {
        if (StringUtils.isBlank(username)) username = request.getHeader("username");
        if (StringUtils.isBlank(username)) {
            if (StringUtils.isBlank(userId)) userId = request.getHeader("userId");
            if (StringUtils.isBlank(userId)) throw new RuntimeException("参数传递异常");
            username = UserService.getInstance().getUser(userId).getUsername();
        }
        return Response.ok(UserService.getInstance().getUserAccount(username));
    }

    @RequestMapping(
            value = {"/type/{platformType}"},
            method = {RequestMethod.PUT}
    )
    @ResponseBody
    public Response updateEachPlatformUsers(HttpServletRequest request,
                                            HttpServletResponse response,
                                            @PathVariable("version") String version,
                                            @PathVariable("platformType") int type,
                                            @RequestBody UserUpdateBean updateBean) throws Exception {
        return Response.success(UserService.getInstance().updateEachPlatformUsers(UserProductType.fromInteger(type), updateBean));
    }
}
