package com.fr.plugin.decision.api.direct;

import com.finebi.burger.api.bean.configuration.table.BusinessTableBean;
import com.finebi.burger.api.response.configuration.pack.CheckedTablePackage;
import com.finebi.burger.impl.service.helper.TablePackagePathHelper;
import com.finebi.burger.impl.service.helper.UserInfoHelper;
import com.finebi.common.api.cell.PackageCell;
import com.finebi.common.api.cell.SpiderCellCreator;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.authority.structure.GroupPackNode;
import com.finebi.common.api.cell.user.authority.structure.TableNode;
import com.finebi.common.api.helper.pack.PackageGroupHelper;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.table.FineAnalysisTable;
import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.common.api.vo.table.FineExcelBusinessTable;
import com.finebi.common.exception.FineEngineException;
import com.finebi.common.exception.conf.pack.FinePackageAbsentException;
import com.finebi.direct.burger.impl.service.helper.table.FineTableUtils;
import com.finebi.direct.burger.impl.service.search.BusinessTableToBusinessTablePerViewBeanVisitor;
import com.finebi.direct.common.api.cell.context.CellCreator;
import com.finebi.utils.CollectionUtils;
import com.finebi.utils.data.NullUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.utils.SpiderHelper;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Collectors;

/**
 * 华为定制
 *
 * 去掉表的可移动鉴权、编辑鉴权、sql表的sql修改鉴权
 * 去掉业务包的编辑鉴权
 * 去掉表的表红检查
 */
public class CheckedTablePackageCreatorForHW {

    private String packageId;
    private UserAuthority authorityUser;
    private boolean needDetails;
    private CheckedTablePackage checkedTablePackage = null;
    private List<FineBusinessTable> businessTables = new ArrayList<>();
    private List<BusinessTableBean> tableBeans = new ArrayList<>();
    private List<BusinessTableBean> errorTable = new ArrayList<>();
    private PackageCell packageCell = CellCreator.getCellProvider().packageCell();
    private BusinessTableToBusinessTablePerViewBeanVisitor beanVisitor;
//    private TableCell tableCell = CellCreator.getCellProvider().tableCell();
//    private FineTableMovableHelper movableHelper;

    public CheckedTablePackageCreatorForHW(String packageId, UserAuthority authorityUser, boolean needDetails) {
        this.packageId = packageId;
        this.authorityUser = authorityUser;
        this.needDetails = needDetails;
//        this.movableHelper = new FineTableMovableHelper(authorityUser, packageCell);
    }

    public CheckedTablePackage create() throws FineEngineException {
        if (null == checkedTablePackage) {
            Optional<FineBusinessPackage> businessPackage = packageCell.getPackageByPackageId(packageId, authorityUser);
            if (!businessPackage.isPresent()) {
                throw new FinePackageAbsentException(packageId);
            }
            checkedTablePackage = new CheckedTablePackage();
            checkedTablePackage.setName(PackageGroupHelper.getPackageShowName(businessPackage.get()));
//            checkedTablePackage.setEditable(authorityUser.isGroupPackManageable(GroupPackNode.pack(packageId)));
            initCheckedTable(businessPackage.get());
        }
        return checkedTablePackage;
    }

    private void initCheckedTable(FineBusinessPackage businessPackage) {
        List<String> tablesName = new ArrayList<>(businessPackage.getTables());
        if (CollectionUtils.isEmpty(tablesName)) {
            return;
        }
        initTableBeans(tablesName);
        tableBeans = businessTables.stream()
//                .filter(table -> {
//                    if (FineTableGrammarCheckHelper.getTableGrammarCheck(table, tableCell)) {
//                        return true;
//                    } else {
//                        Optional<BusinessTableBean> businessTableBean = getBusinessTableBean(table);
//                        businessTableBean.ifPresent(businessTableBean1 -> errorTable.add(businessTableBean1));
//                        return false;
//                    }
//                })
                .map(this::getBusinessTableBean)
                .filter(Optional::isPresent)
                .map(Optional::get)
                .collect(Collectors.toList());
        checkedTablePackage.setTables(tableBeans);
        checkedTablePackage.setErrorTable(errorTable);
    }

    private void initTableBeans(List<String> tablesName) {
        Predicate<String> fineTableIdAllowedPredicate;
        if (isShareToMePackage()) {
            fineTableIdAllowedPredicate = i -> true;
        } else {
            Set<String> allowedTablesByPackageId = authorityUser.tableFilter().usable().getAvailableChildren(GroupPackNode.pack(packageId)).stream().map(TableNode::getId).collect(Collectors.toSet());
            fineTableIdAllowedPredicate = allowedTablesByPackageId::contains;
        }
        initBeanVisitor(tablesName);
        for (String tableName : tablesName) {
            if (!fineTableIdAllowedPredicate.test(tableName)) {
                continue;
            }
            FineBusinessTable table = null;
            try {
                table = FineTableUtils.getTableByName(tableName);
                appendTableBean(table);
            } catch (Exception e) {
                if (!NullUtils.isNull(table)) {
                    appendTableBean(table);
                } else {
                    addSimpleTableBean2ErrorTable(tableName);
                }
            }
        }
    }

    private void initBeanVisitor(List<String> tablesName) {
        LinkedHashMap<String, String> tablePathMap = new LinkedHashMap<>();
        try {
            tablePathMap = TablePackagePathHelper.createTablePathMapWithUser(
                    packageCell.getTablePositions(tablesName),
                    authorityUser.getUser().getInfo().getUserId(),
                    SpiderHelper::isNoGroupId,
                    UserInfoHelper.createUserNameFunction(SpiderCellCreator.getCellProvider().userCellProvider().userCell())
            );
        } catch (FineEngineException e) {
            FineLoggerFactory.getLogger().warn(e, "getTablePositions error");
        }
        this.beanVisitor = new BusinessTableToBusinessTablePerViewBeanVisitor(packageCell, authorityUser, tablePathMap);
    }

    private boolean isShareToMePackage() {
        return packageCell.isShareToMePackage(packageId);
    }

    private void addSimpleTableBean2ErrorTable(String tableName) {
        FineBusinessTable table = null;
        BusinessTableBean tableBean = null;
        try {
            table = FineTableUtils.getTableByNameWithoutFields(tableName);
            tableBean = table.accept(beanVisitor);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().warn(e, "table bean {}  create failed in package ", tableName);
        }
        if (tableBean != null) {
//            tableBean.setEditable(isEditable(table));
            tableBean.setEditable(false);
            errorTable.add(tableBean);
        }
    }

    private void appendTableBean(FineBusinessTable table) {
        try {
            if (!needDetails) {
                table.setFields(null);
                table.setOperatorBeans(null);
                if (table instanceof FineAnalysisTable) {
                    ((FineAnalysisTable) table).setFieldsMap(null);
                }
                if (table instanceof FineExcelBusinessTable) {
                    ((FineExcelBusinessTable) table).setBaseAttach(null);
                    ((FineExcelBusinessTable) table).setExcelFields(null);
                }
            }
            businessTables.add(table);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, "get table bean error {}", table.getName());
        }
    }

    @NotNull
    private Optional<BusinessTableBean> getBusinessTableBean(FineBusinessTable table) {
        try {
            BusinessTableBean tableBean = table.accept(beanVisitor);
//            tableBean.setEditable(isEditable(table));
            // 点击业务包进入表界面直接给到表是否可移动的属性
            // BI-69986：标红的自助数据集移动到按钮不灰化
//            tableBean.setMovable(movableHelper.isTableMovable(packageId));
//            editableHelper.setSqlEditable(tableBean);
            return Optional.of(tableBean);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            return Optional.empty();
        }
    }

//    private boolean isEditable(FineBusinessTable table) {
//        return UserAuthorityExtendHelper.isTableEditable(table.getName(), authorityUser, table.getType() == BICommonConstants.TABLE.ANALYSIS, checkedTablePackage.isEditable(), table.getCreateByWithId());
//    }

}
