package com.fr.plugin.decision.api.direct;

import com.finebi.burger.api.bean.configuration.group.SimpleGroupBean;
import com.finebi.burger.api.response.configuration.pack.CheckedTablePackage;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.identification.UserInfo;
import com.finebi.common.api.cell.user.identification.UserInfoOptional;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.tree.GroupTreeNodeVO;
import com.finebi.common.exception.FineEngineException;
import com.finebi.direct.common.api.cell.context.CellCreator;
import com.finebi.utils.CompareUtils;
import com.fr.log.FineLoggerFactory;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * @author kris
 * @version 6.0
 * Created by kris on 2021/12/31
 */
public class DirectPackageGroupServiceForHW {

    /**
     * 数据列表展示用
     * Common配置
     */
    public static List<SimpleGroupBean> getAllGroupInfo(String groupId, UserAuthority authorityUser) throws FineEngineException {
        Optional<GroupTreeNodeVO> groupTreeNodeVO = new DirectPackageCellForHW().getSubGroupPackageTreeByGroupId(groupId, authorityUser);
        if (groupTreeNodeVO.isPresent()) { // todo: root节点 getGroups
            return Arrays.asList(new FineSpiderGroupHelperForHW(authorityUser).convertToSimpleGroupBeanForHW(groupTreeNodeVO.get()));
        } else {
            FineLoggerFactory.getLogger().error("get group info failed! ");
            return new ArrayList<>();
        }
    }


    public static CheckedTablePackage getPackageTables(String packageId, UserAuthority authorityUser, boolean needFields) throws Exception {
        // todo: 管理员查看非管理员自助数据集和普通场景同时兼容的处理
        UserAuthority realAuthorityUser = getUserAuthority(packageId, authorityUser);
        return new CheckedTablePackageCreatorForHW(packageId, realAuthorityUser, needFields).create();
    }

    private static UserAuthority getUserAuthority(String packageId, UserAuthority authorityUser) throws FineEngineException {
        UserInfoOptional userInfoOptional = authorityUser.getUser().getOptional();
        if (userInfoOptional.isPresent()) {
            UserInfo userInfo = userInfoOptional.getNullable();
            if (userInfo.isAdmin()) {
                String userId = userInfo.getUserId();
                Optional<FineBusinessPackage> packageOptional = CellCreator.getCellProvider().packageCell().getPackageByPackageIdWithoutAuth(packageId);
                boolean isOtherUsersAnalysisPackage = packageOptional
                        .flatMap(businessPackage -> Optional.of(businessPackage.isMyAnalysis() && !CompareUtils.isEqual(userId, businessPackage.getOwner())))
                        .orElse(false);
                if (isOtherUsersAnalysisPackage) {
                    return authorityUser.getHelper().getSuperAuthority();
                }
            }
        }
        return authorityUser;
    }

}
