package com.fr.plugin.decision.api.direct;

import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.authority.structure.GroupPackNode;
import com.finebi.common.api.cell.user.authority.structure.NodeType;
import com.finebi.utils.CollectionUtils;

import java.util.HashSet;
import java.util.Set;

/**
 * 传入一组GroupPackNode和一个UserAuthority
 * 通过requireVisible和requireManageable属性来建造一个带有相应缓存的FineGroupHelperCacheContext对象
 *
 * @author Gaattc.Dang
 * @version 5.1.17
 * Created by Gaattc.Dang on 2021/8/26
 */
public class FineGroupHelperCacheContextBuilderForHW {

    private boolean requireVisible = false;
    private boolean requireManageable = false;
    private Set<GroupPackNode> groupPackNodes;
    private UserAuthority authorityUser;
    private FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextForHW();

    public FineGroupHelperCacheContextBuilderForHW(Set<GroupPackNode> groupPackNodes, UserAuthority authorityUser) {
        this.groupPackNodes = groupPackNodes;
        this.authorityUser = authorityUser;
    }

    public FineGroupHelperCacheContextBuilderForHW requireVisible() {
        requireVisible = true;
        return this;
    }

    public FineGroupHelperCacheContextBuilderForHW requireManageable() {
        requireManageable = true;
        return this;
    }

    public FineGroupHelperCacheContextForHW build() {
        // 根据所需权限进行批量查询并加入缓存
        if (requireVisible) {
            Set<GroupPackNode> visibleNodes = authorityUser.getVisibleGroupPacks(groupPackNodes);
            Set<String> visiblePackIds = new HashSet<>();
            Set<String> visibleGroupIds = new HashSet<>();
            if (CollectionUtils.isNotEmpty(visibleNodes)) {
                visibleNodes.forEach(node -> {
                    if (NodeType.GROUP.equals(node.getType())) {
                        visibleGroupIds.add(node.getId());
                    } else {
                        visiblePackIds.add(node.getId());
                    }
                });
            }
            context.setVisibleGroupIds(visibleGroupIds);
            context.setVisiblePackIds(visiblePackIds);
        }
        if (requireManageable) {
            Set<GroupPackNode> manageableNodes = authorityUser.getManageableGroupPack(groupPackNodes);
            Set<String> manageablePackIds = new HashSet<>();
            Set<String> manageableGroupIds = new HashSet<>();
            if (CollectionUtils.isNotEmpty(manageableNodes)) {
                manageableNodes.forEach(node -> {
                    if (NodeType.GROUP.equals(node.getType())) {
                        manageableGroupIds.add(node.getId());
                    } else {
                        manageablePackIds.add(node.getId());
                    }
                });
            }
            context.setManageableGroupIds(manageableGroupIds);
            context.setManageablePackIds(manageablePackIds);
        }
        return context;
    }
}
