package com.fr.plugin.decision.api.direct;

import com.finebi.burger.api.bean.configuration.group.SimpleGroupBean;
import com.finebi.burger.api.bean.configuration.pack.SimplePackageBean;
import com.finebi.burger.impl.helper.group.FineGroupShowNameHelper;
import com.finebi.burger.impl.helper.pack.FinePackageHelper;
import com.finebi.common.api.cell.SpiderCellCreator;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.authority.structure.GroupPackNode;
import com.finebi.common.api.vo.group.FinePackageGroup;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.tree.GroupTreeNodeVO;
import com.finebi.common.api.vo.tree.PackageTreeNodeVO;
import com.finebi.common.exception.platform.FineUnknownUserException;
import com.finebi.common.exception.platform.FineUserExecuteException;
import com.finebi.spider.common.api.structure.pack.PackType;
import com.finebi.utils.CollectionUtils;
import com.finebi.utils.CompareUtils;
import com.finebi.utils.StringUtils;
import com.fr.locale.InterProviderFactory;
import com.fr.plugin.decision.api.utils.SpiderHelper;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.finebi.common.authority.decision.FineAuthorityConstants.UNKNOWN_USER_TEXT;

/**
 * This class created on 2017/12/28
 *
 * @author Each.Zhang
 */
public class FineSpiderGroupHelperForHW {

    private UserAuthority authorityUser;

    public FineSpiderGroupHelperForHW(UserAuthority authorityUser) {
        this.authorityUser = authorityUser;
    }

    /**
     * 数据列表通用分组层级展示转bean逻辑
     * 1.业务包表数量根据场景不同使用function传入
     * 2.考虑包的使用权限和分组的可视权限
     * 3.对分享的自助数据集/xx的业务包两种特殊类型的业务包不做过滤处理
     */
//    public SimpleGroupBean convertToSimpleGroupBean(GroupTreeNodeVO groupTreeNodeVO, Function<PackageTreeNodeVO, Integer> packageTableCountFunction) throws FineUnknownUserException, FineUserExecuteException {
//        Set<GroupPackNode> nodes = new HashSet<>();
//        getAllGroupAndPackVONodes(groupTreeNodeVO, nodes);
//        FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextBuilderForHW(nodes, authorityUser).requireVisible().requireManageable().build();
//        return getSimpleGroupBean(groupTreeNodeVO, context);
//    }

    // 去掉管理权限
    public SimpleGroupBean convertToSimpleGroupBeanForHW(GroupTreeNodeVO groupTreeNodeVO) throws FineUnknownUserException, FineUserExecuteException {
        Set<GroupPackNode> nodes = new HashSet<>();
        getAllGroupAndPackVONodes(groupTreeNodeVO, nodes);
        FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextBuilderForHW(nodes, authorityUser).requireVisible().build();
        return getSimpleGroupBean(groupTreeNodeVO, context);
    }

    /**
     * 获取所有的分组和业务包节点
     *
     * @param groupTreeNodeVO 根节点VO对象
     * @param nodes           用来存放所有分组和业务包节点的容器
     */
    private void getAllGroupAndPackVONodes(GroupTreeNodeVO groupTreeNodeVO, Set<GroupPackNode> nodes) {
        // root节点时不需要添加
        // 未分组业务包可能需要额外处理
        nodes.add(GroupPackNode.group(groupTreeNodeVO.getFinePackageGroup().getId()));
        groupTreeNodeVO.getSubGroups().forEach(i -> nodes.add(GroupPackNode.group(i.getFinePackageGroup().getId())));
        groupTreeNodeVO.getPackageTreeNodeList().forEach(i -> nodes.add(GroupPackNode.pack(i.getBusinessPackage().getId())));
        for (GroupTreeNodeVO subGroup : groupTreeNodeVO.getSubGroups()) {
            getAllGroupAndPackVONodes(subGroup, nodes);
        }
    }

    private SimpleGroupBean getSimpleGroupBean(GroupTreeNodeVO groupTreeNodeVO,
                                               FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        SimpleGroupBean simpleGroupBean = new SimpleGroupBean();
        List<SimpleGroupBean> subSimpleGroupBeans = new ArrayList<>();
        // root_group是个概念分组，实际没有group信息，因此不需要再做处理
        if (!groupTreeNodeVO.isRootGroup()) {
            // 不同场景下业务包表数量属性不同，写成function传下去
            List<SimplePackageBean> packageList = getSimplePackageBeansWithOutTableCount(groupTreeNodeVO, context);
            simpleGroupBean = createSimpleGroupBean(groupTreeNodeVO.getFinePackageGroup(), packageList, context);
            // 不展示的分组直接返回null
            if (null == simpleGroupBean) {
                return null;
            }
        }
        for (GroupTreeNodeVO subGroupTreeNodeVO : groupTreeNodeVO.getSubGroups()) {
            CollectionUtils.putIfNotNull(subSimpleGroupBeans, getSimpleGroupBean(subGroupTreeNodeVO, context));
        }
        simpleGroupBean.setGroups(subSimpleGroupBeans);
        return simpleGroupBean;
    }

    private List<SimplePackageBean> getSimplePackageBeansWithOutTableCount(GroupTreeNodeVO groupTreeNodeVO,
                                                                           FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        List<SimplePackageBean> packageList = new ArrayList<>();
        List<PackageTreeNodeVO> visiblePacks = groupTreeNodeVO.getPackageTreeNodeList().stream()
                .filter(pack -> context.getVisiblePackIds().contains(pack.getBusinessPackage().getId()))
                .collect(Collectors.toList());
        for (PackageTreeNodeVO packVO : visiblePacks) {
            SimplePackageBean simplePackage = getSimplePackage(packVO.getBusinessPackage());
            // 不需要管理权限
            simplePackage.setEditable(false);
//             不算表数据量
            simplePackage.setTableCount(0);
            CollectionUtils.putIfNotNull(packageList, simplePackage);
        }
        return packageList;
    }

    private SimpleGroupBean createSimpleGroupBean(FinePackageGroup group,
                                                  List<SimplePackageBean> packageList,
                                                  FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        if (!context.getVisibleGroupIds().contains(group.getId())) {
            return null;
        }
        String groupName = FineGroupShowNameHelper.getGroupShowName(
                authorityUser.getUser().getInfo().getUserId(),
                group,
                SpiderHelper.isNoGroupId(group.getId()),
                SpiderCellCreator.getCellProvider().userCellProvider().userCell());
        SimpleGroupBean simpleGroup = new SimpleGroupBean(group.getId(), groupName, group.getInitTime(), packageList);
//        simpleGroup.setEditable(context.getManageableGroupIds().contains(group.getId()));
        simpleGroup.setEditable(false);
        simpleGroup.setMyAnalysis(group.isMyAnalysis());
        return simpleGroup;
    }

    private SimplePackageBean getSimplePackage(FineBusinessPackage pack) {
        SimplePackageBean simplePackage = FinePackageHelper.createSimplePackageBean(pack, authorityUser.getUser().getHelper());
        if (StringUtils.isEmpty(simplePackage.getCreateBy())) {
            simplePackage.setCreateBy(InterProviderFactory.getProvider().getLocText(UNKNOWN_USER_TEXT));
        }
        return simplePackage;
    }

    /**
     * 自助数据集另存为分组层级展示转bean逻辑
     * 只需要提供以下属性：id、name 、editable 、shareAnalysis、myAnalysis
     */
    public SimpleGroupBean convertToSimpleGroupBeanForCopy(GroupTreeNodeVO groupTreeNodeVO) throws FineUnknownUserException, FineUserExecuteException {
        Set<GroupPackNode> nodes = new HashSet<>();
        getAllGroupAndPackVONodes(groupTreeNodeVO, nodes);
        FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextBuilderForHW(nodes, authorityUser).requireVisible().requireManageable().build();
        return getSimpleGroupBeanForCopy(groupTreeNodeVO, context);
    }

    private SimpleGroupBean getSimpleGroupBeanForCopy(GroupTreeNodeVO groupTreeNodeVO,
                                                      FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        SimpleGroupBean simpleGroupBean = new SimpleGroupBean();
        List<SimpleGroupBean> subSimpleGroupBeans = new ArrayList<>();
        if (!groupTreeNodeVO.isRootGroup()) {
            List<SimplePackageBean> packageList = getSimplePackageBeansForCopy(groupTreeNodeVO, context);
            simpleGroupBean = createSimpleGroupBean(groupTreeNodeVO.getFinePackageGroup(), packageList, context);
            if (null == simpleGroupBean) {
                return null;
            }
        }
        for (GroupTreeNodeVO subGroupTreeNodeVO : groupTreeNodeVO.getSubGroups()) {
            CollectionUtils.putIfNotNull(subSimpleGroupBeans, getSimpleGroupBeanForCopy(subGroupTreeNodeVO, context));
        }
        simpleGroupBean.setGroups(subSimpleGroupBeans);
        return simpleGroupBean;
    }

    private List<SimplePackageBean> getSimplePackageBeansForCopy(GroupTreeNodeVO groupTreeNodeVO,
                                                                 FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        List<SimplePackageBean> packageList = new ArrayList<>();
        List<PackageTreeNodeVO> visiblePacks = groupTreeNodeVO.getPackageTreeNodeList().stream()
                .filter(pack -> context.getVisiblePackIds().contains(pack.getBusinessPackage().getId()))
                .collect(Collectors.toList());
        for (PackageTreeNodeVO packVO : visiblePacks) {
            SimplePackageBean simplePackage = getSimplePackageForCopy(packVO.getBusinessPackage());
            simplePackage.setEditable(context.getManageablePackIds().contains(simplePackage.getId()));
            CollectionUtils.putIfNotNull(packageList, simplePackage);
        }
        return packageList;
    }

    private SimplePackageBean getSimplePackageForCopy(FineBusinessPackage pack) {
        return FinePackageHelper.createSimplePackageBeanForCopy(pack);
    }

    /**
     * 递归遍历找到所有业务包id
     * 放入传入的参数packageIds中
     */
    public void getAllPackageIds(GroupTreeNodeVO groupTreeNodeVO, Set<String> packageIds) {
        List<GroupTreeNodeVO> subGroups = groupTreeNodeVO.getSubGroups();
        if (CollectionUtils.isNotEmpty(subGroups)) {
            for (GroupTreeNodeVO subGroup : subGroups) {
                getAllPackageIds(subGroup, packageIds);
            }
        }
        List<PackageTreeNodeVO> packageTreeNodeVOS = groupTreeNodeVO.getPackageTreeNodeList();
        if (CollectionUtils.isNotEmpty(packageTreeNodeVOS)) {
            packageIds.addAll(packageTreeNodeVOS
                    .stream()
                    .map(PackageTreeNodeVO::getBusinessPackage)
                    .filter(Objects::nonNull)
                    .map(FineBusinessPackage::getId)
                    .collect(Collectors.toSet())
            );
        }
    }

    /**
     * 只用于非超管的我的自助数据集界面的转bean处理
     */
    public SimpleGroupBean convertToSimpleGroupBeanForAnalysisGroupManage(GroupTreeNodeVO groupTreeNodeVO) throws FineUnknownUserException, FineUserExecuteException {
        List<SimpleGroupBean> subSimpleGroupBeans = new ArrayList<>();
        // root_group是个概念分组，实际没有group信息，因此不需要再做处理
        List<SimplePackageBean> packageList = getSimplePackageBeansForAnalysisGroupManage(groupTreeNodeVO);
        for (GroupTreeNodeVO subGroupTreeNodeVO : groupTreeNodeVO.getSubGroups()) {
            CollectionUtils.putIfNotNull(subSimpleGroupBeans, convertToSimpleGroupBeanForAnalysisGroupManage(subGroupTreeNodeVO));
        }
        return getSimpleGroupBeanForAnalysisManage(groupTreeNodeVO.getFinePackageGroup(), packageList, subSimpleGroupBeans);
    }

    private List<SimplePackageBean> getSimplePackageBeansForAnalysisGroupManage(GroupTreeNodeVO groupTreeNodeVO) throws FineUnknownUserException, FineUserExecuteException {
        List<SimplePackageBean> packageList = new ArrayList<>();
        for (PackageTreeNodeVO packageTreeNodeVO : groupTreeNodeVO.getPackageTreeNodeList()) {
            // 过滤非自己的分享业务包
            if (packageTreeNodeVO.getBusinessPackage().getPackType() == PackType.SHARE
                    && !CompareUtils.isEqual(packageTreeNodeVO.getBusinessPackage().getOwner(), authorityUser.getUser().getInfo().getUserId())) {
                continue;
            }
            // 过滤没有表的业务包
            if (CompareUtils.isEqual(packageTreeNodeVO.getBusinessPackage().getTables().size(), 0)) {
                continue;
            }
            SimplePackageBean simplePackage = getSimplePackage(packageTreeNodeVO.getBusinessPackage());
            CollectionUtils.putIfNotNull(packageList, simplePackage);
        }
        return packageList;
    }

    private SimpleGroupBean getSimpleGroupBeanForAnalysisManage(FinePackageGroup group, List<SimplePackageBean> packageList, List<SimpleGroupBean> subGroupBeans) throws FineUnknownUserException, FineUserExecuteException {
        // 分组下无可显示的业务包则不显示
        if (CollectionUtils.isEmpty(packageList) && CollectionUtils.isEmpty(subGroupBeans)) {
            return null;
        }
        String groupName = FineGroupShowNameHelper.getGroupShowName(
                authorityUser.getUser().getInfo().getUserId(),
                group,
                SpiderHelper.isNoGroupId(group.getId()),
                SpiderCellCreator.getCellProvider().userCellProvider().userCell());
        SimpleGroupBean simpleGroup = new SimpleGroupBean(group.getId(), groupName, group.getInitTime(), packageList);
        simpleGroup.setMyAnalysis(group.isMyAnalysis());
        simpleGroup.setGroups(subGroupBeans);
        return simpleGroup;
    }

    /**
     * 表"移动"分组列表获取
     * 1.筛去自己所在业务包
     * 2.普通业务包的表筛去自助数据集分组
     */
    public SimpleGroupBean convertToSimpleGroupBeanForTableMove(GroupTreeNodeVO groupTreeNodeVO, FineBusinessPackage currentPackage) throws FineUnknownUserException, FineUserExecuteException {
        Set<GroupPackNode> nodes = new HashSet<>();
        getAllGroupAndPackVONodes(groupTreeNodeVO, nodes);
        FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextBuilderForHW(nodes, authorityUser).requireManageable().build();
        return getSimpleGroupBeanForTableMove(groupTreeNodeVO, currentPackage, context);
    }

    private SimpleGroupBean getSimpleGroupBeanForTableMove(GroupTreeNodeVO groupTreeNodeVO,
                                                           FineBusinessPackage currentPackage,
                                                           FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        if (!currentPackage.isMyAnalysis() && groupTreeNodeVO.getFinePackageGroup().isMyAnalysis()) {
            return null;
        }
        List<SimpleGroupBean> subSimpleGroupBeans = new ArrayList<>();
        List<SimplePackageBean> packageList = getSimplePackageBeansForTableMove(groupTreeNodeVO, currentPackage, context);
        for (GroupTreeNodeVO subGroupTreeNodeVO : groupTreeNodeVO.getSubGroups()) {
            CollectionUtils.putIfNotNull(subSimpleGroupBeans, getSimpleGroupBeanForTableMove(subGroupTreeNodeVO, currentPackage, context));
        }
        return getSimpleGroupBeanForAnalysisManage(groupTreeNodeVO.getFinePackageGroup(), packageList, subSimpleGroupBeans);
    }

    private List<SimplePackageBean> getSimplePackageBeansForTableMove(GroupTreeNodeVO groupTreeNodeVO,
                                                                      FineBusinessPackage currentPackage,
                                                                      FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        List<SimplePackageBean> packageList = new ArrayList<>();
        for (PackageTreeNodeVO packageTreeNodeVO : groupTreeNodeVO.getPackageTreeNodeList()) {
            // 过滤没有管理权限的业务包和当前表所在的业务包
            if (!context.getManageablePackIds().contains(packageTreeNodeVO.getBusinessPackage().getId())
                    || CompareUtils.isEqual(packageTreeNodeVO.getBusinessPackage().getId(), currentPackage.getId())) {
                continue;
            }
            CollectionUtils.putIfNotNull(packageList, getSimplePackage(packageTreeNodeVO.getBusinessPackage()));
        }
        return packageList;
    }

    /**
     * 权限设置-配置表的行权限展示转bean逻辑
     * 1.只显示当前表和当前表的主表所在的所有层级
     * 2.考虑包的使用权限和分组的可视权限
     */
    public SimpleGroupBean convertToSimpleGroupBeanForTableRowAuthorityManage(GroupTreeNodeVO groupTreeNodeVO, Function<PackageTreeNodeVO, Integer> packageTableCountFunction) throws FineUnknownUserException, FineUserExecuteException {
        Set<GroupPackNode> nodes = new HashSet<>();
        getAllGroupAndPackVONodes(groupTreeNodeVO, nodes);
        FineGroupHelperCacheContextForHW context = new FineGroupHelperCacheContextBuilderForHW(nodes, authorityUser).requireManageable().build();
        return getSimpleGroupBeanForTableRowAuthorityManage(groupTreeNodeVO, packageTableCountFunction, context);
    }

    private SimpleGroupBean getSimpleGroupBeanForTableRowAuthorityManage(GroupTreeNodeVO groupTreeNodeVO,
                                                                         Function<PackageTreeNodeVO, Integer> packageTableCountFunction,
                                                                         FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        List<SimpleGroupBean> subSimpleGroupBeans = new ArrayList<>();
        // root_group是个概念分组，实际没有group信息，因此不需要再做处理
        List<SimplePackageBean> packageList = getSimplePackageBeansForTableRowAuthorityManage(groupTreeNodeVO, packageTableCountFunction, context);
        for (GroupTreeNodeVO subGroupTreeNodeVO : groupTreeNodeVO.getSubGroups()) {
            CollectionUtils.putIfNotNull(subSimpleGroupBeans, getSimpleGroupBeanForTableRowAuthorityManage(subGroupTreeNodeVO, packageTableCountFunction, context));
        }
        return getSimpleGroupBeanForAnalysisManage(groupTreeNodeVO.getFinePackageGroup(), packageList, subSimpleGroupBeans);
    }

    private List<SimplePackageBean> getSimplePackageBeansForTableRowAuthorityManage(GroupTreeNodeVO groupTreeNodeVO,
                                                                                    Function<PackageTreeNodeVO, Integer> packageTableCountFunction,
                                                                                    FineGroupHelperCacheContextForHW context) throws FineUnknownUserException, FineUserExecuteException {
        List<SimplePackageBean> packageList = new ArrayList<>();
        for (PackageTreeNodeVO packageTreeNodeVO : groupTreeNodeVO.getPackageTreeNodeList()) {
            // 过滤自助数据集下的业务包
            if (packageTreeNodeVO.getBusinessPackage().isMyAnalysis()) {
                continue;
            }
            // 过滤没有可用主表的业务包
            if (CompareUtils.isEqual(packageTableCountFunction.apply(packageTreeNodeVO), 0)) {
                continue;
            }
            SimplePackageBean simplePackage = getSimplePackage(packageTreeNodeVO.getBusinessPackage());
            try {
                simplePackage.setEditable(context.getManageablePackIds().contains(simplePackage.getId()));
                simplePackage.setTableCount(packageTableCountFunction.apply(packageTreeNodeVO));
            } catch (Exception ignore) {
            }
            CollectionUtils.putIfNotNull(packageList, simplePackage);
        }
        return packageList;
    }
}
