package com.fr.plugin.decision.api.direct.service;

import com.finebi.burger.api.bean.configuration.table.AnalysisTableBean;
import com.finebi.burger.api.bean.configuration.table.etl.operator.SelectFieldBean;
import com.finebi.burger.api.bean.configuration.table.etl.operator.select.SelectFieldBeanItem;
import com.finebi.burger.api.bean.configuration.table.etl.operator.select.SelectFieldPackageItem;
import com.finebi.burger.api.bean.configuration.table.field.FineBusinessField;
import com.finebi.burger.api.request.configuration.table.etl.SimpleAnalysisTableAddRequestBean;
import com.finebi.burger.api.request.configuration.table.etl.SimpleSourceTableInfoRequestBean;
import com.finebi.common.api.cell.PackageCell;
import com.finebi.common.api.cell.TableCell;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.cell.user.identification.UserInfo;
import com.finebi.common.api.vo.table.FineAnalysisTable;
import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.common.exception.FineEngineException;
import com.finebi.foundation.constant.BIConfConstants;
import com.finebi.foundation.utils.ServiceFieldIdUtils;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2022/1/26 10:57
 * @Description TODO
 * @Version 10.0
 **/
public class XAnalysisTableServiceHelper {

    public static AnalysisTableBean createAnalysisTableBean(SimpleAnalysisTableAddRequestBean requestBean,
                                                            PackageCell packageCell,
                                                            TableCell tableCell,
                                                            UserAuthority authorityUser
    ) throws FineEngineException {
        AnalysisTableBean tableBean = new AnalysisTableBean();
        String tableName = requestBean.getTableName();
        String packId = requestBean.getPackageId();
        SimpleSourceTableInfoRequestBean sourceTableInfo = requestBean.getSourceTableInfo();
        Map<String, String> sourceFieldTransferNameMap = getSourceTable(sourceTableInfo.getTableName(), tableCell)
                .getFields()
                .stream()
                .collect(Collectors.toMap(FineBusinessField::getName, FineBusinessField::getTransferName));
        UserInfo userInfo = authorityUser.getUser().getInfo();

        tableBean.setName(tableName);
        tableBean.setTransferName(tableName);
        tableBean.setPack(packId);
        tableBean.setDataAnalysis(
                packageCell.isAnalysisPackage(packId) ||
                        userInfo.isBIDataAnalysisUser()
        );
        tableBean.setResultToIdentifyNameMap(new LinkedHashMap<>());
        tableBean.setLimitType(tableBean.isDataAnalysis() ? BIConfConstants.CONF.PREVIEW_CAL_LIMIT.ALL : BIConfConstants.CONF.PREVIEW_CAL_LIMIT.PART);
        tableBean.setCreateBy(userInfo.getUserName());

        // 需要在基础属性设置完后,再进行步骤的设置
        setSelectFieldOperatorAndResultNameMap(tableBean, sourceTableInfo, sourceFieldTransferNameMap);
        return tableBean;
    }

    private static FineBusinessTable getSourceTable(String tableName, TableCell tableCell) throws FineEngineException {
        FineBusinessTable sourceTable = new FineAnalysisTable();
        sourceTable.setFields(new ArrayList<>());
        sourceTable.setName(tableName);
        Optional<FineBusinessTable> businessTable = tableCell.getSingleTableWithoutAuth(tableName);
        if (businessTable.isPresent()) {
            sourceTable = businessTable.get();
        }
        return sourceTable;
    }

    private static void setSelectFieldOperatorAndResultNameMap(AnalysisTableBean tableBean,
                                                               SimpleSourceTableInfoRequestBean sourceTableInfo,
                                                               Map<String, String> fieldTransferNameMap
    ) {
        String sourceTableName = sourceTableInfo.getTableName();
        List<String> fieldNames = sourceTableInfo.getFieldNames();
        String tableName = tableBean.getName();
        SelectFieldBean selectFieldBean = new SelectFieldBean();
        List<SelectFieldBeanItem> value = new ArrayList<>();
        LinkedHashMap<String, String> resultToIdentifyNameMap = new LinkedHashMap<>();
        // 若 fieldNames 为空，默认全选
        if (fieldNames == null || fieldNames.isEmpty()) {
            fieldNames = new ArrayList<>();
            fieldNames.addAll(fieldTransferNameMap.keySet());
        }
        for (String fieldName : fieldNames) {
            SelectFieldBeanItem beanItem = new SelectFieldBeanItem();
            beanItem.setField(ServiceFieldIdUtils.createFineFieldId(sourceTableName, fieldName));
            beanItem.setTableName(sourceTableName);
            beanItem.setCommonTable(Collections.singletonList(sourceTableName));
            beanItem.setPath(new ArrayList<>());
            String alias = fieldTransferNameMap.getOrDefault(fieldName, fieldName);
            beanItem.setSelectFieldId(ServiceFieldIdUtils.createFineFieldId(tableName, alias));
            beanItem.setPack(new SelectFieldPackageItem());
            value.add(beanItem);
            resultToIdentifyNameMap.put(alias, alias);
        }
        selectFieldBean.setValue(value);
        selectFieldBean.setRoutersMap(new LinkedHashMap<>());

        tableBean.setOperators(Collections.singletonList(selectFieldBean));
        tableBean.setResultToIdentifyNameMap(resultToIdentifyNameMap);
    }
}
