package com.fr.plugin.decision.api.direct.service;

import com.finebi.common.api.listener.ReportChangeListener;
import com.finebi.common.authority.api.FineUserUtils;
import com.finebi.common.exception.conf.FineNoAuthException;
import com.finebi.common.exception.conf.dashboard.FineDashboardDuplicateNameException;
import com.finebi.common.exception.conf.dashboard.FineReportDuplicateException;
import com.finebi.common.exception.conf.dashboard.FineReportLockedException;
import com.finebi.common.exception.conf.dashboard.FineReportNullNameException;
import com.finebi.dashboard.api.bean.plate.PlateStyleBean;
import com.finebi.dashboard.api.bean.report.ReportBean;
import com.finebi.dashboard.api.bean.report.ReportDirBean;
import com.finebi.dashboard.api.bean.report.style.LayoutRatioBean;
import com.finebi.dashboard.api.bean.widget.style.TemplateStyle;
import com.finebi.dashboard.api.cell.*;
import com.finebi.dashboard.api.response.report.dashboard.ReportIndexBean;
import com.finebi.dashboard.api.service.FineReportSessionService;
import com.finebi.dashboard.api.vo.ReportConfVO;
import com.finebi.dashboard.impl.service.exception.FineReportSaveException;
import com.finebi.dashboard.impl.service.helper.CataLogHelper;
import com.finebi.dashboard.impl.service.helper.ReportIndexConverter;
import com.finebi.utils.CollectionUtils;
import com.finebi.utils.CompareUtils;
import com.finebi.utils.RandomUtils;
import com.finebi.utils.StringUtils;
import com.fr.decision.authority.data.User;
import com.fr.decision.finebi.data.report.FineHangoutType;
import com.fr.decision.finebi.data.report.ReportIndex;
import com.fr.decision.finebi.data.report.ReportPublicLink;
import com.fr.general.GeneralUtils;
import com.fr.locale.InterProviderFactory;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.service.APIService;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

/**
 * @Author JianYe.Wang
 * @Data 2022/1/20 14:18
 * @Description TODO
 * @Version 10.0
 **/
public class XFineDashBoardServiceImpl {

    private static volatile XFineDashBoardServiceImpl service;

    private XFineDashBoardServiceImpl() {
    }

    public static XFineDashBoardServiceImpl getInstance() {
        if (null == service) {
            synchronized (XFineDashBoardServiceImpl.class) {
                if (null == service) {
                    service = new XFineDashBoardServiceImpl();
                }
            }
        }

        return service;
    }

    private ReportIndexCell getReportIndexCell() {
        return DashboardCellCreator.reportIndex();
    }

    private ReportBeanCell getReportBeanCell() {
        return DashboardCellCreator.reportBean();
    }

    private ReportTableCell getReportTableCell() {
        return DashboardCellCreator.reportTable();
    }

    private ReportTXCell getReportTX() {
        return DashboardCellCreator.reportTX();
    }

    /**
     * 新建仪表板文件夹
     * @param dirBean
     * @param userId
     * @return
     * @throws Exception
     */
    public ReportIndexBean createNewReportFolder(ReportDirBean dirBean, String userId) throws Exception {
        CataLogHelper cataLogHelper = new CataLogHelper(userId);
        String newName = dirBean.getName();
        ReportIndex index = cataLogHelper.createNewReportFolder(dirBean, userId);
        List<ReportIndex> childReportIndexList = this.getReportIndexCell().findReportIndexByPid(index.getpId());
        for (ReportIndex reportIndex : childReportIndexList) {
            if (CompareUtils.isEqual(dirBean.isFolder(), reportIndex.isFolder())
                    && CompareUtils.isEqual(newName, reportIndex.getName())) {
                throw new FineDashboardDuplicateNameException("DuplicateNameException ReportIndex:" + newName);
            }
        }
        // 校验完同级目录后可以直接修改name，不需要二次rename
        index.setName(newName);
        this.getReportTX().add((new ReportConfVO.Builder(index.getId())).setReportIndex(index).build());
        return ReportIndexConverter.convert(index);
    }


    /**
     * 创建仪表板
     * @param dirBean
     * @param userId
     * @return
     * @throws Exception
     */
    public ReportIndexBean createNewReport(ReportDirBean dirBean, String userId) throws Exception {
        return ReportIndexConverter.convert(this.createNewReportSave(dirBean, userId));
    }

    @NotNull
    private ReportIndex createNewReportSave(ReportDirBean dirBean, String userId) throws Exception {
        if (!this.getReportAuthCell().haveCreateReportAuth(userId)) {
            throw new FineNoAuthException(InterProviderFactory.getProvider().getLocText("BI-Design_No_Auth_Create_Report"));
        } else if (StringUtils.isEmpty(dirBean.getName())) {
            throw new FineReportNullNameException("report named null, reportId = {}", dirBean.getReportId());
        } else {
            CataLogHelper cataLogHelper = new CataLogHelper(userId);
            String pid = cataLogHelper.getParentIdByReportDirBean(dirBean);
            ReportBean initBean = this.createEmptyReportBean();
            initBean.setReportName(dirBean.getName());
            if (this.hasDuplicateName(dirBean.getName(), dirBean.isFolder(), pid)) {
                throw new FineReportDuplicateException(dirBean.getName());
            }

            ReportIndex index = this.createEmptyReportIndex(userId, dirBean.getName(), initBean.getReportId(), pid);
            String path = this.getRelativePathByIndex(index);
            List<String> reportUseTables = this.getReportTableCell().getReportUseTables(initBean);
            boolean success = this.getReportTX().add((new ReportConfVO.Builder(index.getId())).setReportIndex(index).setReportBean(initBean).setPath(path).setTables(reportUseTables).build());
            if (!success) {
                throw new FineReportSaveException(index.getId());
            }

            ReportChangeListener.addEvent(CollectionUtils.set(index.getId()));
            return index;
        }
    }

    private ReportAuthCell getReportAuthCell() {
        return DashboardCellCreator.reportAuthCell();
    }

    private ReportBean createEmptyReportBean() {
        ReportBean initBean = new ReportBean();
        TemplateStyle style = new TemplateStyle();
        style.setPredictionStyles(new ArrayList());
        style.setStyle(new PlateStyleBean());
        initBean.setTemplateStyle(style);
        initBean.setLayoutRatio(new LayoutRatioBean());
        initBean.setFreeLayoutRatio(new LayoutRatioBean());
        String reportId = RandomUtils.getUUID();
        initBean.setReportId(reportId);
        initBean.setWidgets(new HashMap());
        initBean.setLinkageGroup(new ArrayList());
        initBean.setInitTime(System.currentTimeMillis());
        return initBean;
    }

    private boolean hasDuplicateName(String indexName, boolean isFolder, String parentId) throws Exception {
        List<ReportIndex> reportIndexList = this.getReportIndexCell().findSimilarReportIndex(parentId, indexName, isFolder);
        return CollectionUtils.isNotEmpty(reportIndexList);
    }


    private ReportIndex createEmptyReportIndex(String userId, String reportIndexName, String reportId, String pid) {
        ReportIndex index = new ReportIndex();
        index.setId(reportId);
        index.setName(reportIndexName);
        index.setInitTime(System.currentTimeMillis());
        index.setLastUpdateTime(System.currentTimeMillis());
        index.setFolder(false);
        index.setCreateBy(userId);
        index.setPublicLink(new ReportPublicLink(index.getId()));
        index.setpId(pid);
        index.setHangout(FineHangoutType.NORMAL);
        return index;
    }

    public String getRelativePathByIndex(ReportIndex index) {
        return this.getReportBeanCell().getRelativePathByIndex(index);
    }

}
