package com.fr.plugin.decision.api.excel;

import com.finebi.foundation.api.bean.BeanHelper;
import com.finebi.lock.FineLockFactory;
import com.fr.io.utils.ResourceIOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.utils.LogUtils;
import com.fr.stable.StableUtils;

import java.util.HashSet;
import java.util.Set;
import java.util.concurrent.locks.Lock;

/**
 * @Author JianYe.Wang
 * @Data 2021/10/25 16:06
 * @Description TODO
 * @Version 10.0
 **/
public class ExcelReportSelector {

    private static volatile ExcelReportSelector service;
    private static final String FILE_NAME = "excelReportIndex.json";
    public final static String EXCEL_FILE_PATH = "excelModules";

    private ExcelReportSelector() {
    }

    public static ExcelReportSelector getInstance() {
        if (null == service) {
            synchronized (ExcelReportSelector.class) {
                if (null == service) {
                    service = new ExcelReportSelector();
                }
            }
        }

        return service;
    }

    public boolean isExcelReport(String reportId) {
        return getReportIds().contains(reportId);
    }

    public Set<String> getReportIds() {
        try {
            String filePath = StableUtils.pathJoin(EXCEL_FILE_PATH, FILE_NAME);
            if (!ResourceIOUtils.exist(filePath)) {
                filePath = StableUtils.pathJoin("assets", filePath);
            }
            if (!ResourceIOUtils.exist(filePath)) {
                LogUtils.error("load excel json file error!");
                return new HashSet<>();
            }
            byte[] bytes = ResourceIOUtils.readBytes(filePath);
            ExcelReportIndexConf excelReportIndexConf = BeanHelper.excludeNullMapper().readValue(bytes, ExcelReportIndexConf.class);
            Set reportIds = excelReportIndexConf.getReportIds() == null ? new HashSet<>() : excelReportIndexConf.getReportIds();
            LogUtils.info("ExcelReports:{}", reportIds);
            return reportIds;
        } catch (Exception e) {
            LogUtils.error(e, "PublicLinkExportSettingDao getAll error ! ");
            return new HashSet<>();
        }
    }

}
