package com.fr.plugin.decision.api.service;

import com.finebi.activate.foundation.StableManager;
import com.finebi.common.api.vo.group.FinePackageGroup;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.common.exception.FineEngineException;
import com.finebi.dashboard.api.bean.report.ReportDirBean;
import com.finebi.dashboard.api.bean.report.WidgetReportBean;
import com.finebi.dashboard.api.bean.widget.table.TableWidgetBean;
import com.finebi.dashboard.api.response.report.dashboard.ReportIndexBean;
import com.finebi.dashboard.api.service.FineReportIndexService;
import com.finebi.dashboard.impl.service.helper.ReportIndexConverter;
import com.finebi.direct.burger.api.service.FinePackageService;
import com.finebi.direct.burger.api.service.FineTableService;
import com.finebi.provider.api.decision.FineDecisionAdaptEngineProvider;
import com.finebi.spider.common.api.structure.pack.PackType;
import com.fr.base.TemplateUtils;
import com.fr.concurrent.NamedThreadFactory;
import com.fr.decision.AdaptUtils;
import com.fr.decision.authority.AuthorityContext;
import com.fr.decision.authority.data.*;
import com.fr.decision.authority.data.personnel.DepRole;
import com.fr.decision.cache.DecisionCacheProxy;
import com.fr.decision.finebi.bean.BiReportBean;
import com.fr.decision.finebi.data.BiReportExpandRecord;
import com.fr.decision.finebi.data.report.ReportIndex;
import com.fr.decision.webservice.annotation.RoleCheckerType;
import com.fr.decision.webservice.bean.authority.PrivilegeBean;
import com.fr.decision.webservice.bean.authority.PrivilegeDetailBean;
import com.fr.decision.webservice.bean.data.transfer.TransferBIPackageBean;
import com.fr.decision.webservice.bean.entry.EntryBean;
import com.fr.decision.webservice.bean.entry.builder.EntryBeanBuilders;
import com.fr.decision.webservice.bean.pack.BiPackageRowFilterBean;
import com.fr.decision.webservice.bean.user.RoleBean;
import com.fr.decision.webservice.bean.user.UserBean;
import com.fr.decision.webservice.utils.UserSourceFactory;
import com.fr.decision.webservice.utils.WebServiceUtils;
import com.fr.decision.webservice.v10.dashboard.BiDashBoardService;
import com.fr.decision.webservice.v10.entry.cache.EntryTreeCache;
import com.fr.decision.webservice.v10.entry.cache.EntryTreeNodeCache;
import com.fr.decision.webservice.v10.user.CustomRoleService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.context.PluginContexts;
import com.fr.plugin.decision.api.bean.BIAuthorityBean;
import com.fr.plugin.decision.api.bean.DashboardEntryEditBean;
import com.fr.plugin.decision.api.bean.dashboard.ReportIndexExpandBean;
import com.fr.plugin.decision.api.bean.RowFilterBean;
import com.fr.plugin.decision.api.bean.consanguinity.*;
import com.fr.plugin.decision.api.bean.user.UserUpdateDepRoleBean;
import com.fr.plugin.decision.api.excel.ExcelReportSelector;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.RestrictionFactory;
import com.fr.third.fasterxml.jackson.core.JsonProcessingException;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.beanutils.BeanUtils;

import java.io.IOException;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/7/27 17:36
 * @Description TODO
 * @Version 10.0
 **/
public class APIService {

    private static volatile APIService service;

    private APIService() {
    }

    public static APIService getInstance() {
        if (null == service) {
            synchronized (APIService.class) {
                if (null == service) {
                    service = new APIService();
                }
            }
        }

        return service;
    }

    /**
     * 构造创建仪表板 bean
     *
     * @param name     新建仪表板名称
     * @param parentId 新建仪表板父id
     * @return
     */
    public WidgetReportBean constructWidgetReport(String name, String parentId) throws IOException {
        WidgetReportBean widgetReportBean = new WidgetReportBean();
        ReportDirBean reportDirBean = new ReportDirBean();
        reportDirBean.setName(name);
        reportDirBean.setParentId(parentId);
        widgetReportBean.setDest(reportDirBean);
        widgetReportBean.setWidget(constructTableWidget());
        return widgetReportBean;
    }

    /**
     * 构造TableWidgetBean，根据前端 bi.constant.design.widget.conf 组件
     *
     * @return
     * @throws IOException
     */
    public TableWidgetBean constructTableWidget() throws IOException {
        String constantStr = "{\"wId\":\"\",\"type\":1,\"name\":\"-\",\"bounds\":{},\"tableName\":\"\",\"fields\":[],\"measures\":[],\"settings\":{\"nameStyleType\":1,\"titleHeight\":25,\"titleBackground\":{\"type\":\"color\",\"color\":\"\",\"imageId\":\"\"},\"widgetBackground\":{\"type\":\"color\",\"color\":\"\",\"imageId\":\"\"},\"gisBackground\":{\"show\":true,\"normal\":true,\"layer\":null,\"zoomIn\":\"放大\",\"zoomOut\":\"缩小\"},\"chartAttr\":{},\"chartStyle\":{\"legend\":{\"enabled\":true,\"fontStyle\":{\"auto\":true,\"fontFamily\":\"\\\"Microsoft YaHei\\\", PingFangSC-Regular\",\"fontSize\":12,\"fontBold\":false,\"italic\":false,\"fontColor\":\"\"},\"borderColor\":\"transparent\",\"position\":\"right\",\"titleEnabled\":true,\"validMapping\":null},\"axis\":{\"enabled\":true,\"type\":\"solid\",\"color\":\"\",\"width\":1},\"hGrid\":{\"enabled\":true,\"type\":\"solid\",\"color\":\"\",\"width\":1},\"vGrid\":{\"enabled\":true,\"type\":\"solid\",\"color\":\"\",\"width\":1},\"background\":{\"enabled\":true},\"adaptive\":{\"enabled\":true,\"type\":\"standard\"}},\"tableAttr\":{\"color\":{\"all\":{\"color\":\"\",\"backgroundColor\":\"\",\"dims\":[],\"conditions\":[]}},\"shape\":{\"all\":{\"shape\":-1,\"dims\":[],\"conditionsList\":[],\"color\":\"\"}},\"filterValue\":{}},\"tableStyle\":{\"form\":1,\"style\":null,\"themeColor\":null,\"metric\":{\"show\":true,\"position\":3,\"type\":2},\"freezeDim\":true,\"freezeNumbers\":1,\"showSequence\":false,\"showRowInfo\":true,\"headerTextWrapper\":false,\"textWrapper\":false,\"expandRowHeader\":false,\"expandColHeader\":false,\"showRowSum\":true,\"showColSum\":true,\"showChildRowSum\":false,\"showChildColSum\":false,\"rowPosition\":\"bottom\",\"colPosition\":\"right\",\"sumRowText\":\"\",\"headerRowHeight\":25,\"rowHeight\":25,\"detailRowCounts\":100,\"rowCounts\":20,\"colCounts\":7,\"totalRows\":1000,\"font\":{\"type\":1,\"header\":{\"fontFamily\":null,\"fontSize\":null,\"bold\":null,\"italic\":null,\"underline\":null,\"fontAlign\":null,\"fontColor\":null},\"body\":{\"fontFamily\":null,\"fontSize\":null,\"bold\":null,\"italic\":null,\"underline\":null,\"dimFontAlign\":null,\"tarFontAlign\":null,\"fontColor\":null}}},\"transmitLinkages\":true},\"view\":{\"10000\":[],\"20000\":[],\"30000\":[]},\"viewAttr\":{\"10000\":{\"type\":1,\"left\":{},\"right\":{}},\"20000\":{\"type\":1},\"30000\":{\"type\":1,\"left\":{},\"right\":{}}},\"showTitle\":true,\"openJump\":true,\"dimensions\":{},\"linkage\":{},\"jump\":[],\"allData\":false,\"drillOrder\":[],\"resultFilter\":[],\"dimensionGroups\":{},\"legendFilter\":null,\"columnSize\":[],\"regionColumnSize\":[],\"equallyDivideColumn\":false,\"uploadedImages\":[]}";
        TableWidgetBean tableWidgetBean = new ObjectMapper().readValue(constantStr.getBytes(), TableWidgetBean.class);
        tableWidgetBean.setTimeStamp(System.currentTimeMillis());
        return tableWidgetBean;
    }

    public String writeLocalStorageAndOpenWindowScriptStr(String userId, String tableName, String reportId, String paramStr) throws Exception {
        StringBuffer buffer = new StringBuffer();
        buffer.append("<script>")
                .append("var last=JSON.parse(localStorage.getItem('.bi.lastSelectedTable'));last=last||{};")
                .append("last['" + userId + "']==undefined?last['" + userId + "']={undefined:'" + tableName + "'}:last['" + userId + "']['undefined']='" + tableName + "';")
                .append("localStorage.setItem('.bi.lastSelectedTable', JSON.stringify(last));")
                .append("location.href=\"")
                .append(TemplateUtils.render("${fineServletURL}"))
                .append("/v5/design/report/")
                .append(reportId)
                .append("/edit")
                .append(StringUtils.isEmpty(paramStr) ? "" : ("?" + paramStr))
                .append("#open_detail\"")
                .append("</script>");
        return buffer.toString();
    }

    /**
     * 基于用户批量更新更新 部门职位、角色信息
     *
     * @param currentUserId 当前登录用户
     * @param bean          更新信息
     * @throws Exception
     */
    public void updateUserDepRoleInfo(String currentUserId, UserUpdateDepRoleBean bean) throws Exception {
        User targetUser = UserService.getInstance().getUserByUserId(bean.getUserId());
        if (null != bean.getAddDepartmentIds()) {
            for (String addDepRoleId : bean.getAddDepartmentIds()) {
                DepRole depRole = WebServiceUtils.parseUniqueDepartmentPostId(addDepRoleId);
                if (RoleCheckerType.DEPARTMENT.checkAuthority(currentUserId, depRole.getDepartmentId())) {
                    Department department = AuthorityContext.getInstance().getDepartmentController().getById(depRole.getDepartmentId());
                    Post post = AuthorityContext.getInstance().getPostController().getById(depRole.getPostId());
                    UserSourceFactory.getInstance().checkSource(targetUser, department);
                    UserSourceFactory.getInstance().checkSource(targetUser, post);
                    AuthorityContext.getInstance().getUserController().addUserToDepartmentAndPost(targetUser.getId(), depRole.getDepartmentId(), depRole.getPostId());
                }
            }
        }
        if (null != bean.getRemoveDepartmentIds()) {
            for (String removeDepRoleId : bean.getRemoveDepartmentIds()) {
                DepRole depRole = WebServiceUtils.parseUniqueDepartmentPostId(removeDepRoleId);
                if (RoleCheckerType.DEPARTMENT.checkAuthority(currentUserId, depRole.getDepartmentId())) {
                    AuthorityContext.getInstance().getUserController().removeUserFromDepartmentAndPost(targetUser.getId(), depRole.getDepartmentId(), depRole.getPostId());
                }
            }
        }
        if (null != bean.getAddRoleIds()) {
            for (String addRoleId : bean.getAddRoleIds()) {
                if (RoleCheckerType.CUSTOM_ROLE.checkAuthority(currentUserId, addRoleId)) {
                    UserSourceFactory.getInstance().checkSource(targetUser, AuthorityContext.getInstance().getCustomRoleController().getById(addRoleId));
                    AuthorityContext.getInstance().getUserController().addUserToCustomRole(targetUser.getId(), addRoleId);
                }
            }
        }
        if (null != bean.getRemoveRoleIds()) {
            for (String removeRoleId : bean.getRemoveRoleIds()) {
                if (RoleCheckerType.CUSTOM_ROLE.checkAuthority(currentUserId, removeRoleId)) {
                    AuthorityContext.getInstance().getUserController().removeUserFromCustomRole(targetUser.getId(), removeRoleId);
                }
            }
        }
    }

    public PrivilegeBean changeBIAuthBean2Privilege(BIAuthorityBean authorityBean) {
        PrivilegeBean privilegeBean = new PrivilegeBean();
        privilegeBean.setIds(Arrays.asList(new String[]{getRealEntityId(authorityBean)}));
        privilegeBean.setValues(getPrivilegeDetailBean(authorityBean).toArray(new PrivilegeDetailBean[0]));
        return privilegeBean;
    }

    private String getRealEntityId(BIAuthorityBean authorityBean) {
        switch (authorityBean.getType()) {
            case "group":
                return "1@@@" + authorityBean.getId();
            case "pack":
                return "2@@@" + authorityBean.getId();
            case "table":
                return "3@@@" + authorityBean.getId();
            case "column":
                return "4@@@" + authorityBean.getId();
        }
        throw new RuntimeException("参数传递异常！");
    }

    private List<PrivilegeDetailBean> getPrivilegeDetailBean(BIAuthorityBean authorityBean) {
        List<PrivilegeDetailBean> privilegeDetailBeans = new ArrayList<>();
        switch (authorityBean.getAuthType()) {
            case 0: // 使用权限
                if (authorityBean.isAuthValue()) {
                    privilegeDetailBeans.add(new PrivilegeDetailBean(201, 2));
                } else {
                    privilegeDetailBeans.add(new PrivilegeDetailBean(201, 1));
                    // 数据集只有使用权限
                    if (authorityBean.hasManageAuth()) privilegeDetailBeans.add(new PrivilegeDetailBean(202, 1));
                }
                break;
            case 1: // 管理权限
                if (authorityBean.isAuthValue()) {
                    privilegeDetailBeans.add(new PrivilegeDetailBean(201, 2));
                    if (authorityBean.hasManageAuth()) privilegeDetailBeans.add(new PrivilegeDetailBean(202, 2));
                } else {
                    privilegeDetailBeans.add(new PrivilegeDetailBean(201, 1));
                }
                break;
        }
        return privilegeDetailBeans;
    }

    public String transferPost2Id(String role) {
        if (StringUtils.isNotBlank(role)) {
            try {
                RoleBean roleBean = CustomRoleService.getInstance().getCustomRole(role);
                if (null != roleBean) return roleBean.getId();
                CustomRole customRole = AuthorityContext.getInstance().getCustomRoleController().findOne(QueryFactory.create().addRestriction(
                        RestrictionFactory.eq(CustomRole.COLUMN_NAME, role)
                ));
                if (null != customRole) return customRole.getId();
            } catch (Exception ignore) {
            }
        }
        throw new RuntimeException("role parameter is null or not exist!");
    }

    public String transferUser2Id(String userInfo) {
        if (StringUtils.isNotBlank(userInfo)) {
            try {
                User user = UserService.getInstance().getUserByUserName(userInfo);
                if (null != user) return user.getId();
                user = AuthorityContext.getInstance().getUserController().getById(userInfo);
                if (null != user) return user.getId();
            } catch (Exception ignore) {
            }
        }
        throw new RuntimeException("user parameter is null or not exist!");
    }

    public BiPackageRowFilterBean constructRowFilter(RowFilterBean rowFilterBean, int roleType, String roleId) throws JsonProcessingException {
        BiPackageRowFilterBean packageRowFilterBean = new BiPackageRowFilterBean();
        packageRowFilterBean.setId(rowFilterBean.getId());
        if (rowFilterBean.getFilter() == null) {
            packageRowFilterBean.setFilter("");
        } else {
            packageRowFilterBean.setFilter(new ObjectMapper().writeValueAsString(rowFilterBean.getFilter().toAuthFilter()));
        }
        packageRowFilterBean.setRoleId(roleId);
        packageRowFilterBean.setRoleType(roleType);
        packageRowFilterBean.setTableId("3@@@" + rowFilterBean.getTableId());
        List<PrivilegeDetailBean> privilegeDetailBeans = new ArrayList<>();
        privilegeDetailBeans.add(new PrivilegeDetailBean(201, 2));
        packageRowFilterBean.setPrivilegeDetailBeanList(privilegeDetailBeans);
        FineLoggerFactory.getLogger().info("row filter: {}", new ObjectMapper().writeValueAsString(packageRowFilterBean));
        return packageRowFilterBean;
    }

    /**
     * 获取业务包的完整路径id，不包含当前业务包id
     *
     * @param packId
     * @return
     * @throws FineEngineException
     */
    public List<String> getPackageFullIdByPackId(String packId) throws FineEngineException {
        FinePackageService packageService = StableManager.getContext().getServiceBean(FinePackageService.class);
        List<String> pathIdList = new ArrayList<>();
        FinePackageGroup lastGroup = packageService.getGroupByPackageId(packId);
        pathIdList.add(lastGroup.getId());
        while (!StringUtils.equals(lastGroup.getPid(), "__root_group__")) {
            lastGroup = packageService.getGroupById(lastGroup.getPid());
            pathIdList.add(lastGroup.getId());
        }
        Collections.reverse(pathIdList);
        return pathIdList;
    }

    /**
     * 获取业务包的完整路径名称
     *
     * @param packId
     * @return
     * @throws FineEngineException
     */
    public List<String> getPackFullNameByPackId(String packId) throws FineEngineException {
        FinePackageService packageService = StableManager.getContext().getServiceBean(FinePackageService.class);
        FineBusinessPackage businessPackage = packageService.getPackageById(packId);
        List<String> pathIdList = new ArrayList<>();
        pathIdList.add(businessPackage.getName());
        FinePackageGroup lastGroup = packageService.getGroupByPackageId(packId);
        pathIdList.add(lastGroup.getName());
        while (!StringUtils.equals(lastGroup.getPid(), "__root_group__")) {
            lastGroup = packageService.getGroupById(lastGroup.getPid());
            pathIdList.add(lastGroup.getName());
        }
        Collections.reverse(pathIdList);
        return pathIdList;
    }

    /**
     * 获取仪表板的完整路径id
     *
     * @param reportId 仪表板id
     * @param topId    仪表板创建者id，根文件目录的父id是仪表板创建id
     * @return
     * @throws Exception
     */
    public List<String> getReportFullPathById(String reportId, String topId) throws Exception {
        FineReportIndexService reportIndexService = StableManager.getContext().getServiceBean(FineReportIndexService.class);
        ReportIndex reportIndex = reportIndexService.getReportIndexById(reportId).get();
        List<String> pathIdList = new ArrayList<>();
        while (!StringUtils.equals(reportIndex.getpId(), topId)) {
            reportIndex = reportIndexService.getReportIndexById(reportIndex.getpId()).get();
            pathIdList.add(reportIndex.getId());
        }
        Collections.reverse(pathIdList);
        return pathIdList;
    }

    /**
     * 获取仪表板完整路径名称
     *
     * @param reportId 仪表板id
     * @param topId    仪表板创建者id
     * @return
     * @throws Exception
     */
    public List<String> getReportFullPathNameById(String reportId, String topId) throws Exception {
        FineReportIndexService reportIndexService = StableManager.getContext().getServiceBean(FineReportIndexService.class);
        ReportIndex reportIndex = reportIndexService.getReportIndexById(reportId).get();
        List<String> pathNameList = new ArrayList<>();
        while (!StringUtils.equals(reportIndex.getpId(), topId)) {
            reportIndex = reportIndexService.getReportIndexById(reportIndex.getpId()).get();
            pathNameList.add(reportIndex.getName());
        }
        Collections.reverse(pathNameList);
        return pathNameList;
    }

    /**
     * 获取仪表板完整的路径ReportIndex对象
     *
     * @param reportId
     * @param topId
     * @return
     * @throws Exception
     */
    public List<ReportIndex> getReportFullPathIndex(String reportId, String topId) throws Exception {
        FineReportIndexService reportIndexService = StableManager.getContext().getServiceBean(FineReportIndexService.class);
        ReportIndex reportIndex = reportIndexService.getReportIndexById(reportId).get();
        List<ReportIndex> pathIndexList = new ArrayList<>();
        while (!StringUtils.equals(reportIndex.getpId(), topId)) {
            reportIndex = reportIndexService.getReportIndexById(reportIndex.getpId()).get();
            pathIndexList.add(reportIndex);
        }
        Collections.reverse(pathIndexList);
        return pathIndexList;
    }

    public String getReportFullPath4PlatformEntry(String reportId) throws Exception {
        List<ReportIndex> reportIndexList = BiDashBoardService.getInstance().getDashboards(Arrays.asList(reportId));
        List<String> reportFullPath = getReportFullPathNameById(reportIndexList.get(0).getId(), reportIndexList.get(0).getCreateBy());
        UserBean user = UserService.getInstance().getUser(reportIndexList.get(0).getCreateBy());
        String displayName = user.getRealName() + "(" + user.getUsername() + ")";
        reportFullPath.add(0, displayName);
        /*Collections.reverse(reportFullPath);
        reportFullPath.add(displayName);
        Collections.reverse(reportFullPath);*/
        return String.join("/", reportFullPath);
    }

    /**
     * 查询平台目录树某个节点下的所有仪表板节点
     *
     * @param entryId
     * @return
     */
    public List<BiReportBean> getDashboardByEntryID(String entryId) throws Exception {
        Authority authority = AuthorityContext.getInstance().getAuthorityController().getById(entryId);
        List<BiReportBean> dashboardList = new ArrayList<>();
        traverseAuthority(authority, dashboardList);
        return dashboardList;
    }

    private void traverseAuthority(Authority authority, List<BiReportBean> entryBeanList) throws Exception {
        EntryBean entryBean = EntryBeanBuilders.build(authority);
        if (entryBean instanceof BiReportBean) {
            entryBeanList.add((BiReportBean) entryBean);
        }
        QueryCondition condition = QueryFactory.create().addRestriction(RestrictionFactory.eq(Authority.COLUMN_PARENT_ID, authority.getId()));
        List<Authority> childAuthority = AuthorityContext.getInstance().getAuthorityController().find(condition);
        if (!childAuthority.isEmpty()) {
            for (Authority child : childAuthority) {
                traverseAuthority(child, entryBeanList);
            }
        }
    }

    /**
     * 更新目录树中仪表板节点的信息
     *
     * @param entryEditBean
     */
    public void updateDashboardEntry(DashboardEntryEditBean entryEditBean) throws Exception {
        Authority authority = AuthorityContext.getInstance().getAuthorityController().getById(entryEditBean.getId());
        BiReportExpandRecord reportExpandRecord = (BiReportExpandRecord) authority.getExpandDataRecord();
        if (entryEditBean.getText() == null) {
            entryEditBean.setText(authority.getDisplayName());
        }
        if (entryEditBean.getDescription() == null) {
            entryEditBean.setDescription(authority.getDescription());
        }
        if (entryEditBean.getDeviceType() == -1) {
            entryEditBean.setDeviceType(authority.getDeviceType().toInteger());
        }
        entryEditBean.setPath(authority.getPath());
        if (entryEditBean.getTemplateId() == null) {
            entryEditBean.setTemplateId(reportExpandRecord.getTemplateId());
        }
    }

    public void refreshCache() {
        ExecutorService executorService = PluginContexts.currentContext().newSingleThreadExecutor(new NamedThreadFactory("refreshPlatformTree"));
        try {
            executorService.submit(() -> DecisionCacheProxy.getInstance().getDecisionCacheManager().refresh(EntryTreeCache.class.getName()));
            executorService.submit(() -> DecisionCacheProxy.getInstance().getDecisionCacheManager().refresh(EntryTreeNodeCache.class.getName()));
        } finally {
            executorService.shutdown();
        }
    }

    /**
     * 资源导出时获取数据数据信息
     * {@link com.fr.decision.webservice.v10.data.transfer.BiDataTransferService#getAllPackagesTree(String)}
     *
     * @param engineType
     * @return
     * @throws Exception
     */
    public List<TransferBIPackageBean> getAllPackagesTree(String engineType) throws Exception {
        FineDecisionAdaptEngineProvider provider = AdaptUtils.getBIEngineProviderByEngineType(engineType);
        List<FinePackageGroup> groups = provider.getAllGroups();
        List<TransferBIPackageBean> biPackageList = new ArrayList<>();
        for (FinePackageGroup group : groups) {
            if (!group.isMyAnalysis()) {
                String groupId = group.getId();
                TransferBIPackageBean groupBean = new TransferBIPackageBean();
                groupBean.setGroup(true);
                groupBean.setId(groupId);
                groupBean.setText(group.getName());
                groupBean.setModifyTime(group.getInitTime());
                groupBean.setpId(group.getPid());
                biPackageList.add(groupBean);
                List<FineBusinessPackage> packageList = provider.getBusinessPackagesByGroupId(groupId);
                for (FineBusinessPackage businessPackage : packageList) {
                    if (businessPackage.getPackType() != PackType.SHARE) {
                        String packageId = businessPackage.getId();
                        TransferBIPackageBean packageBean = new TransferBIPackageBean();
                        packageBean.setpId(groupId);
                        packageBean.setId(packageId);
                        packageBean.setText(businessPackage.getName());
                        packageBean.setGroup(false);
                        packageBean.setModifyTime(businessPackage.getInitTime());
                        biPackageList.add(packageBean);
                        List<FineBusinessTable> tableList = provider.getTablesWithoutFieldsByPackId(packageId);
                        for (FineBusinessTable fineBusinessTable : tableList) {
                            TransferBIPackageBean tableBean = new TransferBIPackageBean();
                            tableBean.setTable(true);
                            tableBean.setId(fineBusinessTable.getId());
                            tableBean.setpId(packageId);
                            tableBean.setTableType(fineBusinessTable.getType());
                            tableBean.setText(fineBusinessTable.getTransferName() + "(" + fineBusinessTable.getName() + ")");
                            tableBean.setModifyTime(fineBusinessTable.getInitTime());
                            biPackageList.add(tableBean);
                        }
                    }
                }
            }
        }
        return biPackageList;
    }

    public BaseInfo getTemplateInfo(String templateId) {
        try {
            Optional<ReportIndex> reportIndex = StableManager.getContext().getServiceBean(FineReportIndexService.class).getReportIndexById(templateId);
            if (reportIndex.isPresent()) {
                return new BaseInfo(
                        reportIndex.get().getInitTime(),
                        reportIndex.get().getCreateBy(),
                        UserService.getInstance().getUserByUserId(reportIndex.get().getCreateBy()).getUserName(),
                        APIService.getInstance().getReportFullPathById(reportIndex.get().getId(), reportIndex.get().getCreateBy()),
                        String.join("/", APIService.getInstance().getReportFullPathNameById(reportIndex.get().getId(), reportIndex.get().getCreateBy()))
                );
            }
            return null;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    public BaseInfo getTableInfo(String tableName) {
        try {
            FineTableService tableService = StableManager.getContext().getServiceBean(FineTableService.class);
            FinePackageService packageService = StableManager.getContext().getServiceBean(FinePackageService.class);
            FineBusinessTable fineBusinessTable = tableService.getBusinessTableByName(tableName);
            if (fineBusinessTable != null) {
                String packId = packageService.getPackageIdByTableName(fineBusinessTable.getName());
                return new BaseInfo(
                        fineBusinessTable.getInitTime(),
                        fineBusinessTable.getCreateByWithId(),
                        UserService.getInstance().getUserByUserId(fineBusinessTable.getCreateByWithId()).getUserName(),
                        APIService.getInstance().getPackageFullIdByPackId(packId),
                        String.join("/", APIService.getInstance().getPackFullNameByPackId(packId))
                );
            }
            return null;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static FineTableService getTableService() {
        return StableManager.getContext().getServiceBean(FineTableService.class);
    }
}
