package com.fr.plugin.decision.api.service;

import com.finebi.burger.api.bean.configuration.group.SimpleGroupBean;
import com.finebi.burger.api.bean.configuration.pack.SimplePackageBean;
import com.finebi.burger.api.bean.configuration.table.AbstractBusinessTableBean;
import com.finebi.burger.api.bean.configuration.table.AnalysisTableBean;
import com.finebi.burger.api.bean.configuration.table.SqlTableBean;
import com.finebi.burger.api.bean.configuration.table.response.TableSaveResultBean;
import com.finebi.burger.api.request.configuration.table.SqlTableInfoRequestBean;
import com.finebi.burger.api.request.configuration.table.etl.SimpleAnalysisTableAddRequestBean;
import com.finebi.burger.api.response.configuration.pack.CheckedTablePackage;
import com.finebi.burger.api.response.configuration.table.SimpleTableInfoBean;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.provider.ProviderFactory;
import com.finebi.common.api.vo.table.AbstractFineTable;
import com.finebi.common.api.vo.table.FineSQLBusinessTable;
import com.finebi.common.api.vo.table.TableAddResultInfo;
import com.finebi.common.api.vo.table.TableChangeResultDTO;
import com.finebi.common.exception.FineEngineException;
import com.finebi.common.exception.common.FineUnsupportException;
import com.finebi.direct.burger.impl.service.analysis.AnalysisTableServiceHelper;
import com.finebi.direct.burger.impl.service.helper.table.FineTableChangeType;
import com.finebi.direct.burger.impl.service.helper.table.SaveBusinessTableVisitor;
import com.finebi.direct.common.api.cell.context.CellCreator;
import com.finebi.utils.CollectionUtils;
import com.finebi.utils.CompareUtils;
import com.finebi.utils.StringUtils;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.table.TableBaseBean;
import com.fr.plugin.decision.api.direct.DirectPackageGroupServiceForHW;
import com.fr.plugin.decision.api.direct.service.XAnalysisTableServiceHelper;
import com.fr.plugin.decision.api.utils.DirectServiceUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/12/16 15:15
 * @Description TODO
 * @Version 10.0
 **/
public class ConfService {

    private static volatile ConfService service;

    private ConfService() {
    }

    public static ConfService getInstance() {
        if (null == service) {
            synchronized (ConfService.class) {
                if (null == service) {
                    service = new ConfService();
                }
            }
        }

        return service;
    }

    /*
    校验SQL数据集的是否能正常执行
     */
    public SimpleTableInfoBean sqlCheck(SqlTableBean sqlTableBean, UserAuthority authorityUser) throws Exception {
        SqlTableInfoRequestBean sqlTableInfoRequestBean = new SqlTableInfoRequestBean();
        sqlTableInfoRequestBean.setConnectionName(sqlTableBean.getConnectionName());
        sqlTableInfoRequestBean.setPack(sqlTableBean.getPack());
        sqlTableInfoRequestBean.setParamSetting(sqlTableBean.getParamSetting());
        sqlTableInfoRequestBean.setEncryptSql(ProviderFactory.INSTANCE.getSqlCryptProvider().getEncryptSql(sqlTableBean.getSql()));
        return DirectServiceUtils.getSqlPreviewService().getSqlCheckData(sqlTableInfoRequestBean, authorityUser);
    }

    public TableBaseBean getTableBaseInfo(String tableName) throws Exception {
        AbstractFineTable businessTable = (AbstractFineTable) DirectServiceUtils.getTableService().getBusinessTableByName(tableName);
        TableBaseBean baseBean = new TableBaseBean();
        baseBean.setName(tableName);
        baseBean.setTableName(tableName);
        baseBean.setTransferName(businessTable.getTransferName());
        baseBean.setInitTime(businessTable.getInitTime());
        baseBean.setCreateByName(UserService.getInstance().getUserByUserId(businessTable.getCreateByWithId()).getUserName());
        baseBean.setType(businessTable.getType());
        String packId = DirectServiceUtils.getPackageService().getPackageIdByTableName(tableName);
        List<String> fullPathList = APIService.getInstance().getPackFullNameByPackId(packId);
        baseBean.setFullPathName(String.join("/", fullPathList));
        return baseBean;
    }

    /**
     * 新建默认选择列的自助数据集
     * @param requestBean
     * @param authorityUser
     * @return
     * @throws Exception
     */
    public TableSaveResultBean addSimpleAnalysisTable(SimpleAnalysisTableAddRequestBean requestBean, UserAuthority authorityUser) throws Exception {
        long s = System.currentTimeMillis();
        if (CompareUtils.isEqual(requestBean.getTableName(), requestBean.getSourceTableInfo().getTableName())) {
            throw new FineUnsupportException("table can not be its own parent");
        }
        AnalysisTableBean tableBean = XAnalysisTableServiceHelper.createAnalysisTableBean(
                requestBean,
                CellCreator.getCellProvider().packageCell(),
                CellCreator.getCellProvider().tableCell(),
                authorityUser
        );
        long s1 = System.currentTimeMillis();
        SaveBusinessTableVisitor visitor = new SaveBusinessTableVisitor(FineTableChangeType.ADD, authorityUser);
        TableChangeResultDTO tableChangeResultDTO = tableBean.accept(visitor);
        long s2 = System.currentTimeMillis();
        FineLoggerFactory.getLogger().info("addSimpleAnalysisTable time spend {}ms {}ms", s1-s, s2-s);
        return createSaveResultByDTO(tableChangeResultDTO);
    }

    private TableSaveResultBean createSaveResultByDTO(TableChangeResultDTO dto) {
        TableSaveResultBean resultBean = new TableSaveResultBean();
        List<TableAddResultInfo> addInfos = dto.getTableSaveResult().getAddInfos();
        if (CollectionUtils.isNotEmpty(addInfos)) {
            TableAddResultInfo addResultInfo = addInfos.get(0);
            resultBean.setName(addResultInfo.getTableName());
            resultBean.setTransferName(addResultInfo.getTableTransferName());
        }
        return resultBean;
    }

    public void addTableByPackage(List<SimpleGroupBean> list, UserAuthority userAuthority, String st, String et) throws Exception {
        for (SimpleGroupBean simpleGroupBean:list) {
            addTableByPackage(simpleGroupBean.getGroups(), userAuthority, st, et);
            for (SimplePackageBean simplePackageBean:simpleGroupBean.getPacks()) {
                CheckedTablePackage checkedTablePackage = DirectPackageGroupServiceForHW.getPackageTables(simplePackageBean.getId(), userAuthority, false);
                simplePackageBean.setTables(checkedTablePackage.getTables().stream().filter(businessTableBean -> {
                    long ct = ((AbstractBusinessTableBean) businessTableBean).getInitTime();
                    if (StringUtils.isNotBlank(st)) {
                        if (ct <= Long.valueOf(st)) return false;
                    }
                    if (StringUtils.isNotBlank(et)) {
                        if (ct >= Long.valueOf(et)) return false;
                    }
                    return true;
                }).collect(Collectors.toList()));
            }
        }
    }

}
