package com.fr.plugin.decision.api.service;

import com.finebi.dashboard.api.bean.report.MoveReportDirBean;
import com.finebi.dashboard.api.bean.report.ReportDirBean;
import com.finebi.dashboard.api.cell.DashboardCellCreator;
import com.finebi.dashboard.api.cell.ReportIndexCell;
import com.finebi.dashboard.api.response.report.dashboard.ReportIndexBean;
import com.fr.decision.AdaptUtils;
import com.fr.decision.authority.base.constant.AuthorityValue;
import com.fr.decision.authority.data.User;
import com.fr.decision.authority.type.ShareAuthorityType;
import com.fr.decision.finebi.data.report.ReportIndex;
import com.fr.decision.webservice.bean.authority.PrivilegeBean;
import com.fr.decision.webservice.bean.authority.PrivilegeDetailBean;
import com.fr.decision.webservice.exception.general.NoPrivilegeException;
import com.fr.decision.webservice.exception.user.UserNotExistException;
import com.fr.decision.webservice.utils.BiControllerFactory;
import com.fr.decision.webservice.utils.controller.DashBoardController;
import com.fr.decision.webservice.utils.controller.PrivilegeController;
import com.fr.decision.webservice.v10.authority.AuthorityService;
import com.fr.decision.webservice.v10.dashboard.BiDashBoardService;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.dashboard.ReportIndexExpandBean;
import com.fr.plugin.decision.api.bean.dashboard.ShareDashboard;
import com.fr.plugin.decision.api.excel.ExcelReportSelector;
import com.fr.plugin.decision.api.utils.DirectServiceUtils;
import com.fr.plugin.decision.api.utils.LogUtils;
import com.fr.stable.StringUtils;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.Restriction;
import com.fr.stable.query.restriction.RestrictionFactory;
import org.apache.commons.beanutils.BeanUtils;

import javax.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotNull;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/12/24 9:59
 * @Description TODO
 * @Version 10.0
 **/
public class DashboardService {

    private static volatile DashboardService service;
    private final ReportIndexCell reportIndexCell = DashboardCellCreator.reportIndex();

    private DashboardService() {
    }

    public static DashboardService getInstance() {
        if (null == service) {
            synchronized (DashboardService.class) {
                if (null == service) {
                    service = new DashboardService();
                }
            }
        }

        return service;
    }

    /**
     * 给用户分享仪表板
     * @param shareDashboard
     * @param currentUserId
     * @return
     * @throws Exception
     */
    public void shareDashboard(ShareDashboard shareDashboard, String currentUserId) throws Exception {
        if (shareDashboard.getUserId() == null || shareDashboard.getReportId() == null) {
            throw new IllegalArgumentException("entityIds or userIds should not be null");
        }
        //if (this.hasShareUseAuthority(currentUserId) && this.hasShareRoleAuthority(currentUserId, new String[]{shareDashboard.getReportId()})) {
        if (this.hasShareUseAuthority(currentUserId) ) {
            PrivilegeController controller = BiDashBoardService.getInstance().getPrivilegeController(currentUserId);
            ReportIndex reportIndex = AdaptUtils.getBIProvider().getReportIndexById(shareDashboard.getReportId());
            /*User user = UserService.getInstance().getUserByUserId(shareDashboard.getUserId());
            if (user == null) {
                throw new UserNotExistException();
            }*/
            String shareUserId = APIService.getInstance().transferUser2Id(shareDashboard.getUserId());
            if (!StringUtils.equals(currentUserId, reportIndex.getCreateBy())) {
                throw new NoPrivilegeException();
            }
            AuthorityService.getInstance().updateCarrierAuthorityByEntity(currentUserId, controller, "user", shareDashboard.getReportId(), buildSharePrivilege(shareUserId, shareDashboard.isAuthValue()), "dashboard");
            // 模板另存为
            if (shareDashboard.isSaveas()) {
                MoveReportDirBean mv = new MoveReportDirBean();
                ReportDirBean from =  new ReportDirBean();
                from.setReportId(shareDashboard.getReportId());
                ReportDirBean to =  new ReportDirBean();
                to.setName(reportIndex.getName() + "_" + shareUserId);
                to.setParentId(shareUserId);
                mv.setFrom(Arrays.asList(from));
                mv.setTo(to);
                ReportIndexBean saveAs = DirectServiceUtils.getDashboardService().saveAs(mv, shareUserId);
                // todo: 异常 回滚分享
                shareDashboard.setResultReportId(saveAs.getId());
            }
            return;
        }
        throw new NoPrivilegeException();
    }

    private boolean hasShareUseAuthority(String currentUserId) throws Exception {
        DashBoardController dashBoardController = BiControllerFactory.getInstance().getDashBoardController(currentUserId);
        return dashBoardController.doesUserHaveShareUseAuthority(currentUserId);
    }

    private boolean hasShareRoleAuthority(String currentUserId, String[] targetUserIds) throws Exception {
        DashBoardController dashBoardController = BiControllerFactory.getInstance().getDashBoardController(currentUserId);
        Set<String> allowShareUserIds = dashBoardController.doesUserHaveShareRoleAuthority(currentUserId, new HashSet(Arrays.asList(targetUserIds)));
        return allowShareUserIds.containsAll(Arrays.asList(targetUserIds));
    }

    /**
     * 构造分享仪表板 Privilege
     * @param userId 被分享的用户
     * @param authValue 有无权限
     * @return
     */
    private PrivilegeBean buildSharePrivilege(String userId, boolean authValue) {
        PrivilegeBean privilege = new PrivilegeBean();
        privilege.setId(userId);
        privilege.setValues(new PrivilegeDetailBean[]{new PrivilegeDetailBean(ShareAuthorityType.TYPE.toInteger(), authValue ? AuthorityValue.ACCEPT.toInteger(): AuthorityValue.REJECT.toInteger())});
        return privilege;
    }

    /**
     * 获取仪表板信息
     * @param userId    用户id
     * @param onlyFolder    是否仅文件夹
     * @param pId   指定目录
     * @return
     */
    public List<ReportIndex> getReportIndex(String userId, boolean onlyFolder, String pId, String st, String et) throws Exception {
        List<ReportIndex> result = new ArrayList<>();
        addChildReportIndex(userId, onlyFolder, pId, result, st, et);
        return result;
    }

    private void addChildReportIndex(String userId, boolean onlyFolder, String pId, List<ReportIndex> result, String st, String et) throws Exception {
        QueryCondition query = QueryFactory.create().addRestriction(RestrictionFactory.eq("createBy", userId));
        if (onlyFolder) {
            query.addRestriction(RestrictionFactory.eq("folder", true));
        }
        if (!StringUtils.isBlank(pId)) {
            query.addRestriction(RestrictionFactory.eq("pId", pId));
        }
        if (StringUtils.isNotBlank(st)) {
            query.addRestriction(RestrictionFactory.gt("lastUpdateTime", Long.valueOf(st)));
        }
        if (StringUtils.isNotBlank(et)) {
            query.addRestriction(RestrictionFactory.lt("lastUpdateTime", Long.valueOf(et)));
        }

        List<ReportIndex> search = reportIndexCell.find(query);

        if (!StringUtils.isBlank(pId)) {
            for (ReportIndex index : search) {
                result.add(index);
                if (index.isFolder()) {
                    addChildReportIndex(userId, onlyFolder, index.getId(), result, st, et);
                }
            }
        } else {
            result.addAll(search);
        }
    }

    public List<ReportIndexExpandBean> extendReportIndex(List<ReportIndex> reportIndexList, HashMap<String, String> userIdAndNameMap) {
        Set<String> excelReportIdSet = ExcelReportSelector.getInstance().getReportIds();
        return reportIndexList.stream()
                .map(reportIndex -> extendReportIndex(new ReportIndexExpandBean(reportIndex), excelReportIdSet, userIdAndNameMap))
                .collect(Collectors.toList());
    }

    public ReportIndexExpandBean extendReportIndex(ReportIndexExpandBean reportIndex) {
        return extendReportIndex(reportIndex, ExcelReportSelector.getInstance().getReportIds(), new HashMap<>());
    }

    /**
     * 拓展 reportIndex 的属性
     * @param reportIndex
     * @param excelReportIdSet  excel仪表板id集成
     * @param userIdAndNameMap  用户id/name Map，避免批量转换时每次都实时查询
     */
    public ReportIndexExpandBean extendReportIndex(ReportIndexExpandBean reportIndex, Set<String> excelReportIdSet, @NotNull HashMap<String, String> userIdAndNameMap) {
        if (!reportIndex.isFolder()) reportIndex.setExcel(excelReportIdSet.contains(reportIndex.getId()));
        try {
            String username = StringUtils.EMPTY;
            if (userIdAndNameMap.containsKey(reportIndex.getCreateBy())) {
                username = userIdAndNameMap.get(reportIndex.getCreateBy());
            } else {
                User user;
                if ((user = UserService.getInstance().getUserByUserId(reportIndex.getCreateBy())) != null) {
                    username = user.getUserName();
                    userIdAndNameMap.put(reportIndex.getCreateBy(), username);
                }
            }
            reportIndex.setCreateByName(username);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
        }
        return reportIndex;
    }

}
