package com.fr.plugin.decision.api.service;

import com.finebi.common.api.authority.helper.AuthThrowExceptionChecker;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.api.vo.table.TableNameDependenceInfo;
import com.finebi.common.exception.conf.table.FineTableNoAuthException;
import com.finebi.dashboard.api.bean.report.ReportBean;
import com.finebi.dashboard.api.bean.widget.WidgetBean;
import com.finebi.dashboard.api.bean.widget.multitab.MultiTabWidgetBean;
import com.finebi.dashboard.api.cell.DashboardCellCreator;
import com.finebi.dashboard.api.cell.DashboardTableCell;
import com.finebi.dashboard.impl.widget.visitor.WidgetBeanTablesNameVisitor;
import com.finebi.utils.CollectionUtils;
import com.finebi.widget.api.cell.WidgetCellCreator;
import com.finebi.widget.api.cell.WidgetCellProvider;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.api.bean.table.TableBaseBean;
import com.fr.third.fasterxml.jackson.annotation.JsonIgnore;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author JianYe.Wang
 * @Data 2021/12/7 10:57
 * @Description TODO
 * @Version 10.0
 **/
public class DashboardTableFinder {

    private String reportId;
    // 仪表板中直连数据集信息
    private Set<TableBaseBean> directTables = new HashSet<>();
    // 仪表板中直连无权限的数据集信息
    private Set<TableBaseBean> noAuthDirectTables = new HashSet<>();
    // 仪表板中涉及到的所有数据集，以及数据集的父数据集
    @JsonIgnore
    private Set<String> tables = new HashSet<>();
    @JsonIgnore
    private Map<String, TableNameDependenceInfo> engineDependenceInfoMap = new HashMap();

    public DashboardTableFinder() {
    }

    public DashboardTableFinder(String reportId) {
        this.reportId = reportId;
    }

    public void findAllTable() throws Exception {
        Optional<ReportBean> reportBeanByReportId = DashboardCellCreator.reportBean().getReportBeanByReportId(reportId);
        if (reportBeanByReportId.isPresent()) {
            Map<String, WidgetBean> widgets = reportBeanByReportId.get().getWidgets();
            if (CollectionUtils.isNotEmptyMap(widgets)) {
                widgets.values().forEach(widgetBean -> findTableNameFromWidget(widgetBean, widgets));
                directTables.addAll(tables
                        .stream()
                        .map(this::getTableBaseInfoByName)
                        .collect(Collectors.toList()
                        )
                );
            }
        }
    }

    private void findTableNameFromWidget(WidgetBean widgetBean, Map<String, WidgetBean> widgets) {
        // tab组件 WidgetBeanTablesNameVisitor 返回 null
        if (widgetBean instanceof MultiTabWidgetBean) {
            ((MultiTabWidgetBean) widgetBean).getTabs().forEach(widgetTabBean -> {
                widgetTabBean.getWidgets().keySet().forEach(key -> {
                    findTableNameFromWidget(widgets.get(key), widgets);
                });
            });
            return;
        }
        DashboardTableCell tableCell = DashboardCellCreator.table();
        List<String> widgetTables = widgetBean.accept(new WidgetBeanTablesNameVisitor(tableCell));
        // 查找数据集的父数据集来源，权限申请的时候需要找到来源数据集
        widgetTables.forEach(table -> {
            try {
                WidgetCellProvider provider = WidgetCellCreator.getCellProviderByTable(table);
                TableNameDependenceInfo tableNameDependenceInfo;
                if (this.engineDependenceInfoMap.containsKey(provider.getEngineVO().getName())) {
                    tableNameDependenceInfo = this.engineDependenceInfoMap.get(provider.getEngineVO().getName());
                } else {
                    tableNameDependenceInfo = provider.analysisTableCell().getTableDependenceSupplier().get();
                    this.engineDependenceInfoMap.put(provider.getEngineVO().getName(), tableNameDependenceInfo);
                }
                addTableFather(table, tableNameDependenceInfo.getTableFatherNamesMap());
            } catch (Exception e) {
                FineLoggerFactory.getLogger().error(e.getMessage(), e);
            }
        });
    }

    private void addTableFather(String table, Map<String, Set<String>> tableFatherMap) {
        tables.add(table);
        Set<String> fatherTables = tableFatherMap.get(table);
        fatherTables.forEach(ft -> addTableFather(ft, tableFatherMap));
    }

    /*
    获取数据集基本信息
     */
    private TableBaseBean getTableBaseInfoByName(String table) {
        try {
            return ConfService.getInstance().getTableBaseInfo(table);
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e.getMessage(), e);
            throw new RuntimeException(e);
        }
    }

    public void findAllTableWithNoAuth(UserAuthority userAuthority) throws Exception {
        findAllTable();
        directTables.stream().peek(table -> {
            try {
                AuthThrowExceptionChecker.checkTableHasUseAuth(table.getTableName(), userAuthority);
            } catch (FineTableNoAuthException e) {
                noAuthDirectTables.add(table);
            }
        }).count();
    }

    public String getReportId() {
        return reportId;
    }

    public void setReportId(String reportId) {
        this.reportId = reportId;
    }

    public Set<TableBaseBean> getDirectTables() {
        return directTables;
    }

    public void setDirectTables(Set<TableBaseBean> directTables) {
        this.directTables = directTables;
    }

    public Set<TableBaseBean> getNoAuthDirectTables() {
        return noAuthDirectTables;
    }

    public void setNoAuthDirectTables(Set<TableBaseBean> noAuthDirectTables) {
        this.noAuthDirectTables = noAuthDirectTables;
    }
}
