package com.fr.plugin.decision.api.service;

import com.finebi.burger.api.bean.configuration.pack.SimplePackageBean;
import com.finebi.burger.api.bean.configuration.search.PackageTableSearchResultBean;
import com.finebi.burger.api.bean.configuration.search.TableSearchResultBean;
import com.finebi.burger.api.bean.configuration.table.BusinessTableBean;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.common.exception.FineEngineException;
import com.finebi.direct.burger.api.service.FinePackageService;
import com.fr.plugin.decision.api.bean.pack.PackageTableSearchResult;
import com.fr.stable.StringUtils;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 业务包和数据集搜索器， 基于用户、关键字、目录（业务包、分组Id）进行搜索
 * @Author JianYe.Wang
 * @Data 2021/12/16 14:05
 * @Description TODO
 * @Version 10.0
 **/
public class PackageTableSearcher {

    private FinePackageService packageService;
    private List<SimplePackageBean> packs = new ArrayList<>();
    private List<BusinessTableBean> tables = new ArrayList<>();
    private LinkedHashMap<String, String> packPathMap = new LinkedHashMap<>();
    private LinkedHashMap<String, String> tablePathMap = new LinkedHashMap<>();
    private LinkedHashMap<String, String> packIdPathMap = new LinkedHashMap<>();
    private LinkedHashMap<String, String> tableIdPathMap = new LinkedHashMap<>();

    public static final String DELIMITER = "/";

    public PackageTableSearcher(FinePackageService packageService) {
        this.packageService = packageService;
    }

    /**
     * @param authorityUser
     * @param keyword 搜索关键字
     * @param pId 分组或者业务包ID
     * @return
     * @throws FineEngineException
     */
    public PackageTableSearchResult searchResult(UserAuthority authorityUser, String keyword, String pId) throws FineEngineException {
        searchAllPageResult(authorityUser, keyword);
        filterResultByPid(pId);
        return new PackageTableSearchResult(packs, tables, packPathMap, tablePathMap, packIdPathMap, tableIdPathMap);
    }

    // 查询原始所有分页搜索的汇总
    private void searchAllPageResult(UserAuthority authorityUser, String keyword) throws FineEngineException {
        int page = 1, tableCount = 0;
        PackageTableSearchResultBean pageResult;
        do {
            pageResult = packageService.getAllSearchResult(authorityUser, keyword, page);
            // pack 未分页不需要重复添加
            if (page == 1) {
                packs.addAll(pageResult.getFindPackages());
                packs.addAll(pageResult.getMatchPackages());
                packPathMap = pageResult.getPackIdGroupPathMap();
            }
            tables.addAll(pageResult.getFindTableResult().getTables());
            tables.addAll(pageResult.getMatchTableResult().getTables());
            tablePathMap.putAll(pageResult.getTableNameGroupPathMap());
            tableCount += (getTableSearchResultCount(pageResult.getFindTableResult()) + getTableSearchResultCount(pageResult.getMatchTableResult()));
            page++;
        } while (pageResult.getTableResultSize() > tableCount);
    }

    // 获取数据集搜索结果中的数据集总数
    private int getTableSearchResultCount(TableSearchResultBean resultBean) {
        if (resultBean == null) return 0;
        int count = 0;
        if (resultBean.getTables() != null) count += resultBean.getTables().size();
        if (resultBean.getErrorTables() != null) count += resultBean.getErrorTables().size();
        if (resultBean.getUnavailableTables() != null) count += resultBean.getUnavailableTables().size();
        if (resultBean.getChildTables() != null) count += resultBean.getChildTables().size();
        return count;
    }

    private void filterResultByPid(String pId) throws FineEngineException {
        if (StringUtils.isBlank(pId)) return;
        for (String packId : packPathMap.keySet().toArray(new String[0])) {
            List<String> pathIdList = APIService.getInstance().getPackageFullIdByPackId(packId);
            if (pathIdList.contains(pId) || StringUtils.equals(packId, pId)) {
                packIdPathMap.put(packId, String.join(DELIMITER, pathIdList));
            } else {
                packPathMap.remove(packId);
                packs = packs.stream().filter(e -> !StringUtils.equals(e.getId(), packId)).collect(Collectors.toList());
            }
        }

        for (String table : tablePathMap.keySet().toArray(new String[0])) {
            String packId = packageService.getPackageIdByTableName(table);
            List<String> pathIdList = APIService.getInstance().getPackageFullIdByPackId(packId);
            if (pathIdList.contains(pId) || StringUtils.equals(packId, pId)) {
                pathIdList.add(packId);
                tableIdPathMap.put(table, String.join(DELIMITER, pathIdList));
            } else {
                tablePathMap.remove(table);
                tables = tables.stream().filter(e -> !StringUtils.equals(e.getName(), table)).collect(Collectors.toList());
            }
        }
    }
}
