package com.fr.plugin.decision.api.service.consanguinity;

import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.table.*;
import com.finebi.common.exception.FineEngineException;
import com.finebi.dashboard.api.bean.report.ReportBean;
import com.finebi.dashboard.api.bean.report.consanguinity.*;
import com.finebi.dashboard.api.bean.widget.WidgetBean;
import com.finebi.dashboard.api.bean.widget.multitab.MultiTabWidgetBean;
import com.finebi.dashboard.api.cell.DashboardCellCreator;
import com.finebi.dashboard.api.cell.DashboardTableCell;
import com.finebi.dashboard.impl.widget.visitor.WidgetBeanTablesNameVisitor;
import com.finebi.utils.CollectionUtils;
import com.finebi.widget.api.cell.WidgetCellCreator;
import com.finebi.widget.api.cell.WidgetCellProvider;
import com.finebi.widget.api.cell.conf.TableWidgetCell;
import com.finebi.widget.api.cell.user.FineAuthorityUser;
import com.fr.log.FineLoggerFactory;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;
import java.util.function.BiFunction;
import java.util.stream.Collectors;

/**
 * 添加下对tab组件的支持
 */
public class XReportConsanguinityHelper {
    private Map<String, TableNameDependenceInfo> engineDependenceInfoMap = new HashMap();
    private FineAuthorityUser user;
    private Map<String, WidgetBean> widgets;

    public XReportConsanguinityHelper(FineAuthorityUser user) {
        this.user = user;
    }

    public ReportConsanguinityBean getReportConsanguinityByReportId(String reportId) {
        try {
            Optional<ReportBean> reportBeanByReportId = DashboardCellCreator.reportBean().getReportBeanByReportId(reportId);
            DashboardTableCell tableCell = DashboardCellCreator.table();
            if (reportBeanByReportId.isPresent()) {
                ReportBean reportBean = (ReportBean) reportBeanByReportId.get();
                ReportConsanguinityBean reportConsanguinityBean = new ReportConsanguinityBean();
                reportConsanguinityBean.setReportId(reportId);
                reportConsanguinityBean.setReportName(reportBean.getReportName());
                widgets = reportBean.getWidgets();
                if (CollectionUtils.isNotEmptyMap(widgets)) {
                    List<WidgetConsanguinityBean> widgetConsanguinityBeans = (List) widgets.values().stream().map((widget) -> {
                        return this.getWidgetConsanguinity(tableCell, widget);
                    }).collect(Collectors.toList());
                    reportConsanguinityBean.setWidgets(widgetConsanguinityBeans);
                }

                return reportConsanguinityBean;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error(e, e.getMessage());
        }

        return new ReportConsanguinityBean();
    }

    @NotNull
    private WidgetConsanguinityBean getWidgetConsanguinity(DashboardTableCell tableCell, WidgetBean widget) {
        WidgetConsanguinityBean widgetConsanguinityBean = new WidgetConsanguinityBean();
        widgetConsanguinityBean.setWidgetId(widget.getWid());
        widgetConsanguinityBean.setWidgetName(widget.getName());

        // tab组件 WidgetBeanTablesNameVisitor 返回 null
        if (widget instanceof MultiTabWidgetBean) {
            ((MultiTabWidgetBean) widget).getTabs().forEach(widgetTabBean -> {
                widgetTabBean.getWidgets().keySet().forEach(key -> {
                    getWidgetConsanguinity(tableCell, widgets.get(key));
                });
            });
            return widgetConsanguinityBean;
        }

        List<String> accept = (List) widget.accept(new WidgetBeanTablesNameVisitor(tableCell));
        List<TableConsanguinityBean> collect = (List) accept.stream().map((table) -> {
            return this.getTableConsanguinity(table, new WidgetUsedTableFields(widget, this.user));
        }).filter(Objects::nonNull).collect(Collectors.toList());
        widgetConsanguinityBean.setTables(collect);
        return widgetConsanguinityBean;
    }

    @Nullable
    private TableConsanguinityBean getTableConsanguinity(String table, WidgetUsedTableFields widgetUsedTableFields) {
        try {
            WidgetCellProvider cellProviderByTable = WidgetCellCreator.getCellProviderByTable(table);
            widgetUsedTableFields.getClass();
            return this.getTableConsanguinity(cellProviderByTable, table, widgetUsedTableFields::isUsedTableField);
        } catch (Exception var4) {
            FineLoggerFactory.getLogger().error(var4, var4.getMessage(), new Object[0]);
            return null;
        }
    }

    @Nullable
    private TableConsanguinityBean getTableConsanguinity(WidgetCellProvider provider, String currentTable, BiFunction<String, String, Boolean> useFieldFunction) {
        try {
            Optional<FineBusinessPackage> optional = provider.packageCell().getPackageByTableName(currentTable, this.user);
            if (optional.isPresent()) {
                FineBusinessPackage fineBusinessPackage = (FineBusinessPackage) optional.get();
                TableWidgetCell tableWidgetCell = provider.analysisTableCell();
                Optional<FineBusinessTable> singleTable = tableWidgetCell.getSingleTable(currentTable, this.user);
                if (singleTable.isPresent()) {
                    return (TableConsanguinityBean) ((FineBusinessTable) singleTable.get()).accept(this.getFineTableVisitor(provider, useFieldFunction, fineBusinessPackage));
                }
            }
        } catch (Exception var8) {
            FineLoggerFactory.getLogger().error(var8, "get Origin tables error ! current table name is : " + currentTable + " !", new Object[0]);
        }

        return null;
    }

    @NotNull
    private FineBusinessTableVisitor<TableConsanguinityBean> getFineTableVisitor(final WidgetCellProvider provider, final BiFunction<String, String, Boolean> useFieldFunction, final FineBusinessPackage fineBusinessPackage) {
        return new FineBusinessTableVisitor<TableConsanguinityBean>() {
            public TableConsanguinityBean visit(FineAnalysisTable table) throws Exception {
                AnalysisTableConsanguinityBean tableConsanguinityBean = new AnalysisTableConsanguinityBean();
                tableConsanguinityBean.setFatherTables(getAnalysisOriginTables(provider, table.getName()));
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            private List<FieldConsanguinityBean> createFields(FineBusinessTable table) {
                return (List) (CollectionUtils.isEmpty(table.getFields()) ? new ArrayList() : (List) table.getFields().stream().filter((fineBusinessField) -> {
                    return (Boolean) useFieldFunction.apply(table.getName(), fineBusinessField.getId());
                }).map((fineBusinessField) -> {
                    FieldConsanguinityBean fieldConsanguinityBean = new FieldConsanguinityBean();
                    fieldConsanguinityBean.setFieldId(fineBusinessField.getId());
                    fieldConsanguinityBean.setFieldName(fineBusinessField.getName());
                    fieldConsanguinityBean.setTransferName(fineBusinessField.getTransferName());
                    return fieldConsanguinityBean;
                }).collect(Collectors.toList()));
            }

            public TableConsanguinityBean visit(FineDBBusinessTable table) throws Exception {
                DataBaseTableConsanguinityBean tableConsanguinityBean = new DataBaseTableConsanguinityBean();
                tableConsanguinityBean.setConnectionName(table.getConnectionName());
                tableConsanguinityBean.setDbTableName(table.getTableName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineSQLBusinessTable table) throws Exception {
                SqlTableConsanguinityBean tableConsanguinityBean = new SqlTableConsanguinityBean();
                tableConsanguinityBean.setConnectionName(table.getConnName());
                tableConsanguinityBean.setSql(table.getSql());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineServerBusinessTable table) throws Exception {
                ServerTableConsanguinityBean tableConsanguinityBean = new ServerTableConsanguinityBean();
                tableConsanguinityBean.setServerTableName(table.getSourceName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineExcelBusinessTable table) throws Exception {
                ExcelTableConsanguinityBean tableConsanguinityBean = new ExcelTableConsanguinityBean();
                tableConsanguinityBean.setExcelName(table.getBaseAttach().getFileName());
                tableConsanguinityBean.setPackageId(fineBusinessPackage.getId());
                tableConsanguinityBean.setPackageName(fineBusinessPackage.getName());
                tableConsanguinityBean.setTableName(table.getName());
                tableConsanguinityBean.setTransferName(table.getTransferName());
                tableConsanguinityBean.setFields(this.createFields(table));
                return tableConsanguinityBean;
            }

            public TableConsanguinityBean visit(FineOldETLTable fineOldETLTable) throws Exception {
                throw new UnsupportedOperationException("FineOldETLTable unsupport Report Consanguinity !");
            }
        };
    }

    private Set<TableConsanguinityBean> getAnalysisOriginTables(WidgetCellProvider provider, String currentTable) throws FineEngineException {
        TableNameDependenceInfo tableNameDependenceInfo;
        if (this.engineDependenceInfoMap.containsKey(provider.getEngineVO().getName())) {
            tableNameDependenceInfo = (TableNameDependenceInfo) this.engineDependenceInfoMap.get(provider.getEngineVO().getName());
        } else {
            tableNameDependenceInfo = (TableNameDependenceInfo) provider.analysisTableCell().getTableDependenceSupplier().get();
            this.engineDependenceInfoMap.put(provider.getEngineVO().getName(), tableNameDependenceInfo);
        }

        Set<String> fatherTables = (Set) tableNameDependenceInfo.getTableFatherNamesMap().get(currentTable);
        return (Set) (CollectionUtils.isEmpty(fatherTables) ? new HashSet() : (Set) fatherTables.stream().map((fatherTable) -> {
            return this.getTableConsanguinity(provider, fatherTable, (tableName, fieldId) -> {
                return true;
            });
        }).collect(Collectors.toSet()));
    }
}
